@@ -1,6 +1,11 @@
 #!/usr/bin/env python
 # Thanks to Charles Twardy for this example
-from matplotlib.matlab import *
+#
+#See http://matplotlib.sf.net/examples/legend_demo2.py for an example
+#controlling which lines the legend uses and the order
+
+
+from pylab import *
 
 a = arange(0,3,.02)
 b = arange(0,3,.02)
@@ -12,7 +17,7 @@ d = array(d)
 ax = subplot(111)
 plot(a,c,'k--',a,d,'k:',a,c+d,'k')
 legend(('Model length', 'Data length', 'Total message length'),
-       'upper center')
+       'upper center', shadow=True)
 ax.set_ylim([-1,20])
 ax.grid(0)
 xlabel('Model complexity --->')
@@ -32,8 +37,8 @@ frame  = leg.get_frame()  # the patch.Rectangle instance surrounding the legend
 # see text.Text, lines.Line2D, and patches.Rectangle for more info on
 # the settable properties of lines, text, and rectangles
 frame.set_facecolor(0.80)     # set the frame face color to light gray
-set(ltext, 'fontsize', 'small')    # the legend text fontsize
-set(llines, 'linewidth', 1.5) # the legend linewidth
+set(ltext, fontsize='small')    # the legend text fontsize
+set(llines, linewidth=1.5) # the legend linewidth
 #leg.draw_frame(False)         # don't draw the legend frame
 #savefig('legend_demo')
 show()
