@@ -15,7 +15,7 @@ from matplotlib.transforms import get_bbox_transform, Point, Value, Bbox,\
 if 1:   # load the data
     # data are 256x256 16 bit integers
     dfile = 'data/s1045.ima'
-    im = fromstring(file(dfile, 'rb').read(), Int16).astype(Float)
+    im = fromstring(file(dfile, 'rb').read(), UInt16).astype(Float)
     im.shape = 256, 256
 
 if 1: # plot the MRI in pcolor
