@@ -7,7 +7,7 @@ from numerix import arange, array, asarray, ones, zeros, \
      nonzero, take, Float, log10, logical_and
 
 from artist import Artist
-from cbook import enumerate, silent_list
+from cbook import enumerate, silent_list, popall
 from lines import Line2D, TICKLEFT, TICKRIGHT, TICKUP, TICKDOWN
 from mlab import linspace
 from matplotlib import rcParams
@@ -23,21 +23,22 @@ from text import Text, _process_text_args
 class Tick(Artist):
     """
     Abstract base class for the axis ticks, grid lines and labels
+    
+    1 refers to the bottom of the plot for xticks and the left for yticks
+    2 refers to the top of the plot for xticks and the right for yticks
 
     Publicly accessible attributes
-
+    
       tick1line  : a Line2D instance
       tick2line  : a Line2D instance
       gridline   : a Line2D instance
-      label1      : an Text instance
-      label2      : an Text instance
+      label1     : a Text instance
+      label2     : a Text instance
       gridOn     : a boolean which determines whether to draw the tickline
       tick1On    : a boolean which determines whether to draw the 1st tickline
-                   (left for xtick and bottom for yticks)
       tick2On    : a boolean which determines whether to draw the 2nd tickline
-                   (left for xtick and bottom for yticks)
-      label1On    : a boolean which determines whether to draw tick label
-      label2On    : a boolean which determines whether to draw tick label
+      label1On   : a boolean which determines whether to draw tick label
+      label2On   : a boolean which determines whether to draw tick label
       
       
     """
@@ -73,7 +74,8 @@ class Tick(Artist):
         self._loc = loc
         self._size = size
         self._pad = Value(pad)
-        self._padPixels = self.figure.dpi*self._pad*Value(1/72.0)
+        
+        self._padPixels = self.figure.dpi*self._pad*Value(1/72.0) 
 
 
         self.tick1line = self._get_tick1line(loc)
@@ -233,18 +235,19 @@ class XTick(Tick):
         self._set_artist_props(t)
         return t
         
-    def _get_tick1line(self, loc):
+    def _get_tick1line(self, loc): 
         'Get the default line2D instance'
         # x in data coords, y in axes coords
         l = Line2D( xdata=(loc, loc), ydata=(0, 0),
                     color='k',
+                    linestyle = 'None',
                     antialiased=False,
                     marker = TICKUP,
                     markersize=self._size,
                     )
 
         l.set_transform( blend_xy_sep_transform( self.axes.transData,
-                                                 self.axes.transAxes) )
+                                       self.axes.transAxes) )
         self._set_artist_props(l)
         return l
 
@@ -253,6 +256,7 @@ class XTick(Tick):
         # x in data coords, y in axes coords
         l = Line2D( xdata=(loc, loc), ydata=(1,1),
                        color='k',
+                       linestyle = 'None',
                        antialiased=False, 
                        marker = TICKDOWN,
                        markersize=self._size,
@@ -350,17 +354,18 @@ class YTick(Tick):
         self._set_artist_props(t)
         return t
 
-    def _get_tick1line(self, loc):
+    def _get_tick1line(self, loc): 
         'Get the default line2D instance'
         # x in axes coords, y in data coords
 
         l = Line2D( (0, 0), (loc, loc), color='k',
                     antialiased=False,
                     marker = TICKRIGHT,
+                    linestyle = 'None',
                     markersize=self._size,
                        )
         l.set_transform( blend_xy_sep_transform( self.axes.transAxes,
-                                                 self.axes.transData) )
+                                       self.axes.transData) )
         self._set_artist_props(l)
         return l
     
@@ -370,6 +375,7 @@ class YTick(Tick):
         l = Line2D( (1, 1), (0, 0), color='k',
                     antialiased=False, 
                     marker = TICKLEFT,
+                    linestyle = 'None',
                     markersize=self._size,
                     )
 
@@ -378,7 +384,7 @@ class YTick(Tick):
         self._set_artist_props(l)
         return l
     
-    def _get_gridline(self, loc):
+    def _get_gridline(self, loc): 
         'Get the default line2D instance'
         # x in axes coords, y in data coords
         l = Line2D( xdata=(0,1), ydata=(loc,loc), 
@@ -417,7 +423,12 @@ class YTick(Tick):
         'return the Interval instance for this axis data limits'
         return self.axes.dataLim.intervaly()
 
-    
+
+class Ticker:
+    locator = None
+    formatter = None
+
+
 class Axis(Artist):
 
     """
@@ -434,27 +445,46 @@ class Axis(Artist):
         """
         Artist.__init__(self)
         self.set_figure(axes.figure)
-        
+
         self.axes = axes
+        self.major = Ticker()
+        self.minor = Ticker()
+        #class dummy:
+        #    locator = None
+        #    formatter = None
+        #self.major = dummy()
+        #self.minor = dummy()
 
-        self.set_major_locator( AutoLocator() )
-        self.set_minor_locator( NullLocator() )
+        self.label = self._get_label()
+        self.majorTicks = []
+        self.minorTicks = []
+        
 
-        self.set_major_formatter( ScalarFormatter() )
-        self.set_minor_formatter( NullFormatter() )
+        self.cla()
+        
+    def cla(self):
+        'clear the current axis'
+        self.set_major_locator(AutoLocator())
+        self.set_major_formatter(ScalarFormatter())
+        self.set_minor_locator(NullLocator())
+        self.set_minor_formatter(NullFormatter())
         
         # whether the grids are on
         self._gridOnMajor = rcParams['axes.grid']  
         self._gridOnMinor = False
-
-        self.label = self._get_label()
+        
+        self.label.set_text('')
         self._set_artist_props(self.label)
 
         # build a few default ticks; grow as necessary later; only
         # define 1 so properties set on ticks will be copied as they
         # grow
-        self.majorTicks = [self._get_tick(major=True)  for i in range(1)]
-        self.minorTicks = [self._get_tick(major=False) for i in range(1)]       
+
+        popall(self.majorTicks)
+        popall(self.minorTicks)        
+
+        self.majorTicks.extend([self._get_tick(major=True)  for i in range(1)])
+        self.minorTicks.extend([self._get_tick(major=False) for i in range(1)])
         
     def get_view_interval(self):
         'return the Interval instance for this axis view limits'
@@ -473,11 +503,12 @@ class Axis(Artist):
         if not self.get_visible(): return 
         renderer.open_group(__name__)
         ticklabelBoxes = []
+        ticklabelBoxes2 = []
         
         majorTicks = self.get_major_ticks()
-        majorLocs = self._majorLocator()
-        self._majorFormatter.set_locs(majorLocs)
-        majorLabels = [self._majorFormatter(val, i) for i, val in enumerate(majorLocs)]
+        majorLocs = self.major.locator()
+        self.major.formatter.set_locs(majorLocs)
+        majorLabels = [self.major.formatter(val, i) for i, val in enumerate(majorLocs)]
 
 
         seen = {}
@@ -491,13 +522,17 @@ class Axis(Artist):
             tick.set_label1(label)
             tick.set_label2(label)            
             tick.draw(renderer)
-            extent = tick.label1.get_window_extent(renderer) 
-            ticklabelBoxes.append(extent)
+            if tick.label1On:
+                extent = tick.label1.get_window_extent(renderer) 
+                ticklabelBoxes.append(extent)
+            if tick.label2On:
+                extent = tick.label2.get_window_extent(renderer) 
+                ticklabelBoxes2.append(extent)
 
         minorTicks = self.get_minor_ticks()
-        minorLocs = self._minorLocator()
-        self._minorFormatter.set_locs(minorLocs)
-        minorLabels = [self._minorFormatter(val, i) for i, val in enumerate(minorLocs)]
+        minorLocs = self.minor.locator()
+        self.minor.formatter.set_locs(minorLocs)
+        minorLabels = [self.minor.formatter(val, i) for i, val in enumerate(minorLocs)]
 
 
         for tick, loc, label in zip(minorTicks, minorLocs, minorLabels):
@@ -508,8 +543,12 @@ class Axis(Artist):
             tick.set_label(label)
 
             tick.draw(renderer)  
-            extent = tick.label1.get_window_extent(renderer) 
-            ticklabelBoxes.append(extent)
+            if tick.label1On:
+                extent = tick.label1.get_window_extent(renderer) 
+                ticklabelBoxes.append(extent)
+            if tick.label2On:
+                extent = tick.label2.get_window_extent(renderer) 
+                ticklabelBoxes2.append(extent)
 
 
 
@@ -519,7 +558,7 @@ class Axis(Artist):
         # the actual bbox
 
 
-        self._update_label_postion(ticklabelBoxes)
+        self._update_label_postion(ticklabelBoxes, ticklabelBoxes2)
 
         self.label.draw(renderer)  # memory leak here, vertical text
 
@@ -560,7 +599,7 @@ class Axis(Artist):
 
     def get_ticklocs(self):
         "Get the tick locations in data coordinates as a Numeric array"
-        return self._majorLocator()
+        return self.major.locator()
 
     def _get_tick(self, major):
         'return the default tick intsance'
@@ -569,12 +608,12 @@ class Axis(Artist):
     def _copy_tick_props(self, src, dest):
         'Copy the props from src tick to dest tick'
         if src is None or dest is None: return 
-        dest.label1.copy_properties(src.label1)
-        dest.label2.copy_properties(src.label2)
+        dest.label1.update_from(src.label1)
+        dest.label2.update_from(src.label2)
 
-        dest.tick1line.copy_properties(src.tick1line)
-        dest.tick2line.copy_properties(src.tick2line)        
-        dest.gridline.copy_properties(src.gridline)
+        dest.tick1line.update_from(src.tick1line)
+        dest.tick2line.update_from(src.tick2line)        
+        dest.gridline.update_from(src.gridline)
 
         dest.tick1On = src.tick1On
         dest.tick2On = src.tick2On
@@ -583,24 +622,24 @@ class Axis(Artist):
 
     def get_major_locator(self):
         'Get the locator of the major ticker'
-        return self._majorLocator 
+        return self.major.locator 
 
     def get_minor_locator(self):
         'Get the locator of the minor ticker'
-        return self._minorLocator 
+        return self.minor.locator 
 
     def get_major_formatter(self):
         'Get the formatter of the major ticker'
-        return self._majorFormatter 
+        return self.major.formatter 
 
     def get_minor_formatter(self):
         'Get the formatter of the minor ticker'
-        return self._minorFormatter 
+        return self.minor.formatter 
         
     def get_major_ticks(self):
         'get the tick instances; grow as necessary'
 
-        numticks = len(self._majorLocator())
+        numticks = len(self.major.locator())
 
         if len(self.majorTicks)<numticks:
             # update the new tick label properties from the old
@@ -618,7 +657,7 @@ class Axis(Artist):
 
     def get_minor_ticks(self):
         'get the minor tick instances; grow as necessary'
-        numticks = len(self._minorLocator())
+        numticks = len(self.minor.locator())
         if len(self.minorTicks)<numticks:
             protoTick = self.minorTicks[0]
             for i in range(numticks-len(self.minorTicks)):
@@ -656,18 +695,18 @@ class Axis(Artist):
 Set the formatter of the major ticker
 
 ACCEPTS: A Formatter instance"""
-        self._majorFormatter = formatter
-        self._majorFormatter.set_view_interval( self.get_view_interval() )
-        self._majorFormatter.set_data_interval( self.get_data_interval() )
+        self.major.formatter = formatter
+        self.major.formatter.set_view_interval( self.get_view_interval() )
+        self.major.formatter.set_data_interval( self.get_data_interval() )
 
     def set_minor_formatter(self, formatter):
         """
 Set the formatter of the minor ticker
 
 ACCEPTS: A Formatter instance"""
-        self._minorFormatter = formatter
-        self._minorFormatter.set_view_interval( self.get_view_interval() )
-        self._minorFormatter.set_data_interval( self.get_data_interval() )
+        self.minor.formatter = formatter
+        self.minor.formatter.set_view_interval( self.get_view_interval() )
+        self.minor.formatter.set_data_interval( self.get_data_interval() )
 
 
     def set_major_locator(self, locator):
@@ -675,9 +714,9 @@ ACCEPTS: A Formatter instance"""
 Set the locator of the major ticker
 
 ACCEPTS: a Locator instance"""
-        self._majorLocator = locator
-        self._majorLocator.set_view_interval( self.get_view_interval() )
-        self._majorLocator.set_data_interval( self.get_data_interval() )
+        self.major.locator = locator
+        self.major.locator.set_view_interval( self.get_view_interval() )
+        self.major.locator.set_data_interval( self.get_data_interval() )
 
 
     def set_minor_locator(self, locator):
@@ -685,9 +724,9 @@ ACCEPTS: a Locator instance"""
 Set the locator of the minor ticker
 
 ACCEPTS: a Locator instance"""
-        self._minorLocator = locator
-        self._minorLocator.set_view_interval( self.get_view_interval() )
-        self._minorLocator.set_data_interval( self.get_data_interval() )
+        self.minor.locator = locator
+        self.minor.locator.set_view_interval( self.get_view_interval() )
+        self.minor.locator.set_data_interval( self.get_data_interval() )
                
     def set_ticklabels(self, ticklabels, *args, **kwargs):
         """
@@ -725,11 +764,11 @@ ACCEPTS: sequence of floats"""
 
     def pan(self, numsteps):
         'Pan numticks (can be positive or negative)'
-        self._majorLocator.pan(numsteps)
+        self.major.locator.pan(numsteps)
 
     def zoom(self, direction):
         "Zoom in/out on axis; if direction is >0 zoom in, else zoom out"
-        self._majorLocator.zoom(direction)
+        self.major.locator.zoom(direction)
         
 class XAxis(Axis):
     __name__ = 'xaxis'
@@ -750,25 +789,54 @@ class XAxis(Axis):
                                                      identity_transform() ))
 
         self._set_artist_props(label)
+        self.label_position='bottom'
         return label
 
+    def get_label_position(self):
+        """
+Return the label position (top or bottom)
+        """
+        return self.label_position
 
-    def _update_label_postion(self, bboxes):
+    def set_label_position(self, position):
+        """
+Set the label position (top or bottom)
+
+ACCEPTS: [ 'top' | 'bottom' ]
+        """
+        assert position == 'top' or position == 'bottom'
+        if position == 'top':
+            self.label.set_verticalalignment('bottom')
+        else:
+            self.label.set_verticalalignment('top')
+        self.label_position=position
+
+    def _update_label_postion(self, bboxes, bboxes2):
         """
         Update the label position based on the sequence of bounding
         boxes of all the ticklabels 
         """
 
         x,y = self.label.get_position()
-        if not len(bboxes):
-            bottom = self.axes.bbox.ymin()
-        else:
+        if self.label_position == 'bottom':
+            if not len(bboxes):
+                bottom = self.axes.bbox.ymin()
+            else:
 
-            bbox = bbox_all(bboxes)
-            bottom = bbox.ymin()
+                bbox = bbox_all(bboxes)
+                bottom = bbox.ymin()
             
-        self.label.set_position( (x, bottom-self.LABELPAD*self.figure.dpi.get()/72.0))
-        
+            self.label.set_position( (x, bottom-self.LABELPAD*self.figure.dpi.get()/72.0))
+
+        else:
+            if not len(bboxes2):
+                top = self.axes.bbox.ymax()
+            else:
+
+                bbox = bbox_all(bboxes2)
+                top = bbox.ymax()
+            
+            self.label.set_position( (x, top+self.LABELPAD*self.figure.dpi.get()/72.0))
 
     def tick_top(self):
         'use ticks only on top'
@@ -822,24 +890,54 @@ class YAxis(Axis):
                                                      self.axes.transAxes) )
 
         self._set_artist_props(label)
+        self.label_position='left'
         return label
 
-    def _update_label_postion(self, bboxes):
+    def get_label_position(self):
+        """
+Return the label position (left or right)
+        """
+        return self.label_position
+
+    def set_label_position(self, position):
+        """
+Set the label position (left or right)
+
+ACCEPTS: [ 'left' | 'right' ]
+        """
+        assert position == 'left' or position == 'right'
+        if position == 'right':
+            self.label.set_horizontalalignment('left')
+        else:
+            self.label.set_horizontalalignment('right')
+        self.label_position=position
+
+    def _update_label_postion(self, bboxes, bboxes2):
         """
         Update the label position based on the sequence of bounding
-        boxes overlaps of all the ticklabels that overlap the current
-        ticklabel.  overlaps are the bounding boxes of the ticklabels
+        boxes of all the ticklabels 
         """
 
         x,y = self.label.get_position()
-        if not len(bboxes):
-            left = self.axes.bbox.xmin()
+        if self.label_position == 'left':
+            if not len(bboxes):
+                left = self.axes.bbox.xmin()
+            else:
+
+                bbox = bbox_all(bboxes)
+                left = bbox.xmin()
+            
+            self.label.set_position( (left-self.LABELPAD*self.figure.dpi.get()/72.0, y))
+
         else:
-            bbox = bbox_all(bboxes)
-            left = bbox.xmin()
-        self.label.set_position((left - self.figure.dpi.get()*self.LABELPAD/72.0,y))
-
+            if not len(bboxes2):
+                right = self.axes.bbox.xmax()
+            else:
 
+                bbox = bbox_all(bboxes2)
+                right = bbox.xmax()
+            
+            self.label.set_position( (right+self.LABELPAD*self.figure.dpi.get()/72.0, y))
 
     def tick_right(self):
         'use ticks only on right'
