@@ -76,7 +76,7 @@ from matplotlib.numerix import array, Float
 
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.backend_bases import RendererBase,\
-     GraphicsContextBase, FigureManagerBase, FigureCanvasBase, error_msg
+     GraphicsContextBase, FigureManagerBase, FigureCanvasBase
 
 from matplotlib.cbook import enumerate, is_string_like, exception_to_str
 from matplotlib.figure import Figure
@@ -112,6 +112,7 @@ class RendererAgg(RendererBase):
                                     debug=False)
         self.draw_polygon = self._renderer.draw_polygon
         self.draw_rectangle = self._renderer.draw_rectangle
+        self.draw_path = self._renderer.draw_path
         self.draw_lines = self._renderer.draw_lines
         self.draw_markers = self._renderer.draw_markers
         self.draw_image = self._renderer.draw_image
@@ -204,7 +205,7 @@ class RendererAgg(RendererBase):
 
             font.load_char(ord(s))
         else:
-            font.set_text(str(s), angle)
+            font.set_text(s, angle)
         font.draw_glyphs_to_bitmap()
 
         self._renderer.draw_text(font, int(x), int(y), gc)
@@ -221,7 +222,7 @@ class RendererAgg(RendererBase):
                 s, self.dpi.get(), prop.get_size_in_points())
             return width, height
         font = self._get_agg_font(prop)
-        font.set_text(str(s), 0.0)  # the width and height of unrotated string
+        font.set_text(s, 0.0)  # the width and height of unrotated string
         w, h = font.get_width_height()
         w /= 64.0  # convert from subpixels
         h /= 64.0
@@ -307,6 +308,10 @@ class FigureCanvasAgg(FigureCanvasBase):
         """
         if __debug__: verbose.report('FigureCanvasAgg.draw', 'debug-annoying')
 
+        renderer = self.get_renderer()
+        self.figure.draw(renderer)
+
+    def get_renderer(self):
         l,b,w,h = self.figure.bbox.get_bounds()
         key = w, h, self.figure.dpi.get()
         try: self._lastKey, self.renderer
@@ -316,8 +321,8 @@ class FigureCanvasAgg(FigureCanvasBase):
         if need_new_renderer:
             self.renderer = RendererAgg(w, h, self.figure.dpi)
             self._lastKey = key
-        self.figure.draw(self.renderer)
-
+        return self.renderer
+    
     def tostring_rgb(self):
         if __debug__: verbose.report('FigureCanvasAgg.tostring_rgb', 'debug-annoying')
         return self.renderer.tostring_rgb()
@@ -399,7 +404,7 @@ class FigureCanvasAgg(FigureCanvasBase):
                 ps = self.switch_backends(FigureCanvasPS)
                 ps.print_figure(filename, dpi, facecolor, edgecolor, orientation)
             else:
-                error_msg('Do not know know to handle extension *%s' % ext)
+                raise IOError('Do not know know to handle extension *%s' % ext)
 
         # restore the original figure properties
         self.figure.dpi.set(origDPI)
