@@ -6,11 +6,7 @@ Cairo is a vector graphics library with cross-device output support.
 Features of Cairo:
  * anti-aliasing
  * alpha channel 
- * in-memory image buffers
- * save image files:
-   - PNG
-   - PostScript (50% complete)
-   - PDF        (in development)
+ * saves image files as PNG, PostScript, PDF
 
 http://cairographics.org
 Requires (in order, all available from Cairo website):
@@ -23,26 +19,24 @@ Naming Conventions
 """
 
 from __future__ import division
-
-import os
-import sys
+import os, sys, warnings
 def _fn_name(): return sys._getframe(1).f_code.co_name
 
-from matplotlib import verbose
+import matplotlib.agg as agg
+from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
+     FigureManagerBase, FigureCanvasBase
+from matplotlib.cbook      import enumerate, izip
+from matplotlib.figure     import Figure
+from matplotlib.mathtext   import math_parse_s_ft2font
 from matplotlib.numerix import asarray, pi, fromstring, UInt8, zeros
 import matplotlib.numerix as numerix
-from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
-     FigureManagerBase, FigureCanvasBase, error_msg
-from matplotlib.cbook import enumerate
-from matplotlib.figure import Figure
 from matplotlib.transforms import Bbox
-from matplotlib.mathtext import math_parse_s_ft2font
 
 import cairo
 
 version_required = (0,1,4)
 try: cairo.version_info
-except AttributeError:
+except AttributeError:  # remove at version 0.1.5 or 6
    backend_version = 'Unknown'
 else:
    if cairo.version_info < version_required:
@@ -52,12 +46,15 @@ else:
    backend_version = cairo.version
    del version_required
 
+HAVE_CAIRO_NUMPY = True
+try:
+   import cairo.numpy
+except:
+   HAVE_CAIRO_NUMPY = False
+
 
 DEBUG = False
 
-# the true dots per inch on the screen; should be display dependent
-PIXELS_PER_INCH = 96
-
 # Image formats that this backend supports - for print_figure()
 IMAGE_FORMAT          = ['eps', 'png', 'ps', 'svg']
 #IMAGE_FORMAT          = ['eps', 'pdf', 'png', 'ps', 'svg'] # pdf not ready yet
@@ -92,12 +89,11 @@ class RendererCairo(RendererBase):
         }
     
 
-    def __init__(self, matrix, dpi):
+    def __init__(self, dpi):
         """width, height - the canvas width, height. Is not necessarily
         the same as the surface (pixmap) width, height
         """
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
-        self.matrix   = matrix
         self.dpi      = dpi
         self.text_ctx = cairo.Context()
 
@@ -106,13 +102,18 @@ class RendererCairo(RendererBase):
         self.surface = cairo.gtk.surface_create_for_drawable (pixmap)
 
     def _set_width_height(self, width, height):
-        self.width    = width
-        self.height   = height
+        self.width  = width
+        self.height = height
+        self.matrix_flipy = cairo.Matrix (d=-1, ty=self.height)
+        # use matrix_flipy for ALL rendering?
+        # - problem with text? - will need to switch matrix_flipy off, or do a font transform?
 
     def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2):
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
-        # cairo draws circular arcs (width=height)
-        # could curve_to() and draw a spline instead?
+        # draws circular arcs where width=height
+        # FIXME
+        # to get a proper arc of width/height you can use translate() and scale()
+        # see draw_arc() manual page
         radius = (height + width) / 4
         ctx    = gc.ctx
         ctx.new_path()
@@ -129,9 +130,12 @@ class RendererCairo(RendererBase):
     def draw_image(self, x, y, im, origin, bbox):
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
 
-        try: import cairo.numpy
-        except:
-            verbose.report_error("cairo.numpy module required for draw_image()")
+        if numerix.which[0] == "numarray":
+            warnings.warn("draw_image() currently works for numpy, but not numarray")
+            return
+
+        if not HAVE_CAIRO_NUMPY:
+            warnings.warn("cairo.numpy module required for draw_image()")
             return
 
         # bbox - not currently used
@@ -139,7 +143,6 @@ class RendererCairo(RendererBase):
 
         ctx = cairo.Context()
         ctx.set_target_surface (self.surface)
-        ctx.set_matrix (self.matrix)
 
         rows, cols, buf = im.buffer_argb32()  # ARGB32, but colors still wrong
         X = fromstring(buf, UInt8)
@@ -155,12 +158,9 @@ class RendererCairo(RendererBase):
         #X.shape = rows, cols, 4
 
         # ARGB32
-        try:
-           # works for numpy image, not a numarray image
-           surface = cairo.numpy.surface_create_for_array (X)
-        except TypeError, exc:
-            verbose.report_error("%s: %s" % (_fn_name(), exc))
-            return
+
+        # works for numpy X, not numarray X
+        surface = cairo.numpy.surface_create_for_array (X)
 
         # Alternative
         #surface = cairo.surface_create_for_image(buf, cairo.FORMAT_ARGB32, cols, rows) #, stride)
@@ -179,12 +179,20 @@ class RendererCairo(RendererBase):
         ctx.stroke()
 
 
-    def draw_lines(self, gc, x, y):
+    def draw_lines(self, gc, x, y, transform=None):
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
-        y = [self.height - y for y in y]
-        points = zip(x,y)
-        x, y = points.pop(0)
+
+        if transform:
+            if transform.need_nonlinear():
+                x, y = transform.nonlinear_only_numerix(x, y)
+            x, y = transform.numerix_x_y(x, y)
+        
         ctx = gc.ctx
+        matrix_old = ctx.matrix
+        ctx.set_matrix (self.matrix_flipy)
+
+        points = izip(x,y)
+        x, y = points.next()
         ctx.new_path()
         ctx.move_to (x, y)
 
@@ -192,7 +200,58 @@ class RendererCairo(RendererBase):
             ctx.line_to (x, y)
         ctx.stroke()
 
+        ctx.set_matrix (matrix_old)
 
+
+    def draw_markers(self, gc, path, rgbFace, x, y, transform):
+        if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
+
+        ctx = gc.ctx
+
+        if transform.need_nonlinear():
+            x,y = transform.nonlinear_only_numerix(x, y)
+        
+        x, y = transform.numerix_x_y(x, y) # do nonlinear and affine transform
+
+        # TODO - use cairo transform
+        # matrix worked for dotted lines, but not markers in line_styles.py
+        # it upsets/transforms generate_path() ?
+        # need to flip y too, and update generate_path() ?
+        # the a,b,c,d,tx,ty affine which transforms x and y
+        #vec6 = transform.as_vec6_val() # not used (yet)
+        #matrix_old = ctx.matrix
+        #ctx.set_matrix (cairo.Matrix (*vec6))
+
+        path_list = [path.vertex() for i in range(path.total_vertices())]
+
+        def generate_path (path_list):
+           for code, xp, yp in path_list:
+               if code == agg.path_cmd_move_to:
+                  ctx.move_to (xp, -yp)
+               elif code == agg.path_cmd_line_to:
+                  ctx.line_to (xp, -yp)
+               elif code == agg.path_cmd_end_poly:
+                  ctx.close_path()
+
+        for x,y in izip(x,y):
+            ctx.save()
+            ctx.new_path()
+            ctx.translate(x, self.height - y)
+            generate_path (path_list)
+
+            if rgbFace:
+               ctx.save()
+               ctx.set_rgb_color (*rgbFace)
+               # later - set alpha also?                     
+               ctx.fill()
+               ctx.restore() # undo colour change and restore path
+            
+            ctx.stroke()
+            ctx.restore() # undo translate()
+
+        #ctx.set_matrix(matrix_old)
+
+        
     def draw_point(self, gc, x, y):
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
         # render by drawing a 0.5 radius circle
@@ -205,11 +264,14 @@ class RendererCairo(RendererBase):
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
 
         ctx = gc.ctx
+        matrix_old = ctx.matrix
+        ctx.set_matrix (self.matrix_flipy)
+        
         ctx.new_path()
         x, y = points[0]
-        ctx.move_to (x, self.height - y)
+        ctx.move_to (x, y)
         for x,y in points[1:]:
-            ctx.line_to (x, self.height - y)
+            ctx.line_to (x, y)
         ctx.close_path()
 
         if rgbFace:
@@ -218,7 +280,8 @@ class RendererCairo(RendererBase):
             ctx.fill()
             ctx.restore()
         ctx.stroke()
-
+        
+        ctx.set_matrix (matrix_old)
 
     def draw_rectangle(self, gc, rgbFace, x, y, width, height):
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
@@ -233,7 +296,8 @@ class RendererCairo(RendererBase):
         ctx.stroke()
 
 
-    def draw_text(self, gc, x, y, s, prop, angle, ismath=False):    
+    def draw_text(self, gc, x, y, s, prop, angle, ismath=False):
+        # Note: x,y are device/display coords, not user-coords, unlike other draw_* methods
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
 
         if ismath:
@@ -246,9 +310,10 @@ class RendererCairo(RendererBase):
            ctx.select_font (prop.get_name(),
                             self.fontangles [prop.get_style()],
                             self.fontweights[prop.get_weight()])
-           # 1.4 scales font to a similar size to GTK / GTKAgg backends
-           size = prop.get_size_in_points() * self.dpi.get() / PIXELS_PER_INCH * 1.4
 
+           # size = prop.get_size_in_points() * self.dpi.get() / 96.0
+           size = prop.get_size_in_points() * self.dpi.get() / 72.0
+        
            ctx.save()
            if angle:
               ctx.rotate (-angle * pi / 180)
@@ -261,9 +326,12 @@ class RendererCairo(RendererBase):
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
        # mathtext using the gtk/gdk method
 
-        try: import cairo.numpy
-        except:
-            verbose.report_error("cairo.numpy module required for _draw_mathtext()")
+        if numerix.which[0] == "numarray":
+            warnings.warn("_draw_mathtext() currently works for numpy, but not numarray")
+            return
+
+        if not HAVE_CAIRO_NUMPY:
+            warnings.warn("cairo.numpy module required for _draw_mathtext()")
             return
 
         size = prop.get_size_in_points()
@@ -300,6 +368,7 @@ class RendererCairo(RendererBase):
         pa[:,:,2] = int(rgb[2]*255)
         pa[:,:,3] = Xs
 
+        # works for numpy pa, not a numarray pa
         surface = cairo.numpy.surface_create_for_array(pa)
         gc.ctx.translate (x,y)
         gc.ctx.show_surface (surface, imw, imh)
@@ -309,6 +378,7 @@ class RendererCairo(RendererBase):
     def flipy(self):
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
         return True
+        #return False # tried - all draw objects ok except text (and images?) which comes out mirrored!
 
     
     def get_canvas_width_height(self):
@@ -329,8 +399,12 @@ class RendererCairo(RendererBase):
                          self.fontangles [prop.get_style()],
                          self.fontweights[prop.get_weight()])
 
-        # 1.4 scales font to a similar size to GTK / GTKAgg backends
-        size = prop.get_size_in_points() * self.dpi.get() / PIXELS_PER_INCH * 1.4
+        # Cairo (says it) uses 1/96 inch user space units, ref: cairo_gstate.c
+        # but if /96.0 is used the font is too small
+
+        #size = prop.get_size_in_points() * self.dpi.get() / 96.0
+        size = prop.get_size_in_points() * self.dpi.get() / 72.0
+        
         # problem - scale remembers last setting and font can become
         # enormous causing program to crash
         # save/restore prevents the problem
@@ -346,15 +420,13 @@ class RendererCairo(RendererBase):
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
         gc = GraphicsContextCairo (renderer=self)
         gc.ctx.set_target_surface (self.surface)
-        gc.ctx.set_matrix (self.matrix)
         return gc
 
 
     def points_to_pixels(self, points):
         if DEBUG: print 'backend_cairo.RendererCairo.%s()' % _fn_name()
-        # is this correct for cairo? (copied from gtk)
-        return points * PIXELS_PER_INCH/72.0 * self.dpi.get()/72.0
-
+        return points/72.0 * self.dpi.get()
+     
 
 class GraphicsContextCairo(GraphicsContextBase):
     _joind = {
@@ -374,27 +446,34 @@ class GraphicsContextCairo(GraphicsContextBase):
         GraphicsContextBase.__init__(self)
         self.renderer = renderer
         self.ctx = cairo.Context()
-
+        # default is 0.1, raise value to increase performance
+        # (and lower quality)
+        self.ctx.set_tolerance(0.5)
         
     def set_alpha(self, alpha):
         self._alpha = alpha
         self.ctx.set_alpha(alpha)
 
+    #def set_antialiased(self, b):
+        # enable/disable anti-aliasing is not (yet) supported by Cairo
+
 
     def set_capstyle(self, cs):
         if cs in ('butt', 'round', 'projecting'):
             self._capstyle = cs
             self.ctx.set_line_cap (self._capd[cs])
         else:
-            verbose.report_error('Unrecognized cap style.  Found %s' % cs)
+            raise ValueError('Unrecognized cap style.  Found %s' % cs)
 
 
     def set_clip_rectangle(self, rectangle):
-        # Cairo clipping is currently extremely slow
-        # cairo/BUGS lists it as a known bug
+        # Cairo < 0.4.0: clipping is currently extremely slow
+        # Cairo 0.4.0  : pixel-aligned rectangular clip-regions are now faster
         self._cliprect = rectangle
 
         x,y,w,h = rectangle
+        # pixel-aligned clip-regions are faster        
+        x,y,w,h = round(x), round(y), round(w), round(h)
         ctx = self.ctx
         ctx.new_path()
         ctx.rectangle (x, self.renderer.height - h - y, w, h)
@@ -405,6 +484,9 @@ class GraphicsContextCairo(GraphicsContextBase):
         #ctx.stroke()
         #ctx.restore()        
 
+        #ctx.init_clip() # not needed? used unsuccessfully to fix clip problem
+        # when uncomment ctx.clip() it causes problems in line_styles.py
+        # - multiple axes, only the first one has its background drawn
         #ctx.clip ()
         
 
@@ -413,8 +495,8 @@ class GraphicsContextCairo(GraphicsContextBase):
         if dashes == None:
             self.ctx.set_dash([], 0)  # switch dashes off
         else:
-            dashes_pixels = self.renderer.points_to_pixels(asarray(dashes))
-            self.ctx.set_dash(dashes_pixels, offset)
+            self.ctx.set_dash(self.renderer.points_to_pixels(asarray(dashes)),
+                              offset)
         
 
     def set_foreground(self, fg, isRGB=None):
@@ -432,7 +514,7 @@ class GraphicsContextCairo(GraphicsContextBase):
             self._joinstyle = js
             self.ctx.set_line_join(self._joind[js])
         else:
-            verbose.report_error('Unrecognized join style.  Found %s' % js)
+            raise ValueError('Unrecognized join style.  Found %s' % js)
 
 
     def set_linewidth(self, w):
@@ -474,7 +556,7 @@ def print_figure_fn(figure, filename, dpi=150, facecolor='w', edgecolor='w',
             try:
                 fileObject = file(filename,'wb')
             except IOError, exc:
-                verbose.report_error("%s: %s" % (exc.filename, exc.strerror))
+                warnings.warn("%s: %s" % (exc.filename, exc.strerror))
             else:
                 if ext == 'png': _save_png (figure, fileObject)
                 else:            _save_ps_pdf (figure, fileObject, ext, orientation)
@@ -489,8 +571,8 @@ def print_figure_fn(figure, filename, dpi=150, facecolor='w', edgecolor='w',
             fc.print_figure(filename, dpi, facecolor, edgecolor, orientation)
 
         else:
-            verbose.report_error('Format "%s" is not supported.\nSupported formats: %s.' %
-                                 (ext, ', '.join(IMAGE_FORMAT)))
+            warnings.warn('Format "%s" is not supported.\nSupported formats: %s.' %
+                          (ext, ', '.join(IMAGE_FORMAT)))
 
         
 def _save_png (figure, fileObject):
@@ -500,7 +582,7 @@ def _save_png (figure, fileObject):
     ctx = cairo.Context()
     ctx.set_target_png (fileObject, cairo.FORMAT_ARGB32, width, height)
 
-    renderer = RendererCairo (ctx.matrix, figure.dpi)
+    renderer = RendererCairo (figure.dpi)
     renderer._set_width_height(width, height)
     renderer.surface = ctx.target_surface
     figure.draw(renderer)
@@ -538,7 +620,7 @@ def _save_ps_pdf (figure, fileObject, ext, orientation):
         # TODO:
         # add portrait/landscape checkbox to FileChooser
 
-    renderer = RendererCairo (ctx.matrix, figure.dpi)
+    renderer = RendererCairo (figure.dpi)
     renderer._set_width_height(width, height)
     renderer.surface = ctx.target_surface
     figure.draw(renderer)
