@@ -51,6 +51,9 @@ IMAGE_FORMAT          = ['bmp', 'eps', 'jpg', 'png', 'ps', 'svg']
 IMAGE_FORMAT.sort()
 IMAGE_FORMAT_DEFAULT  = 'png'
 
+DBL_BUFFER = True 
+#DBL_BUFFER = False # test to see if switching double buffering off is useful
+
 
 cursord = {
     cursors.MOVE          : gtk.gdk.Cursor(gtk.gdk.FLEUR),
@@ -119,10 +122,6 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
 
         self.set_flags(gtk.CAN_FOCUS)
         self.grab_focus()
-        # widget can take any size, gtk.Window sets default size
-        #self.set_size_request (int (figure.bbox.width()),
-        #                       int (figure.bbox.height()))
-        
         self.set_double_buffered(False)
 
         self.connect('key_press_event',      self.key_press_event)
@@ -176,7 +175,6 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
         return key
 
 
-
     def key_press_event(self, widget, event):
         key = self._get_key(event)
         FigureCanvasBase.key_press_event(self, key)
@@ -205,6 +203,14 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
         self._draw_pixmap = True
         self.expose_event(self, None)
 
+    def draw_idle(self):
+        def idle_draw(*args):
+            self.draw()
+            self._idleID = 0
+            return False
+        if self._idleID==0:
+            self._idleID = gtk.idle_add(idle_draw)
+
 
     def _renderer_init(self):
         """Override by GTK backends to select a different renderer
@@ -212,33 +218,37 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
             _set_pixmap ()
             _set_width_height ()
         that are used by
-            _render_to_pixmap()        
+            _render_figure()        
         """
         self._renderer = RendererGDK (self, self.figure.dpi)
 
 
-    def _render_to_pixmap(self, width, height):
-        """Render the figure to a gdk.Pixmap, is used for
+    def _render_figure(self, width, height):
+        """Render the figure to a gdk.Pixmap, used by expose_event().
+        Is used for
            - rendering the pixmap to display        (pylab.draw)
            - rendering the pixmap to save to a file (pylab.savefig)
-        Should not be overridden
+        Should not be overridden by GTK backends
         """
         if DEBUG: print 'FigureCanvasGTK.%s' % fn_name()
-        create_pixmap = False
-        if width > self._pixmap_width:
-            # increase the pixmap in 10%+ (rather than 1 pixel) steps
-            self._pixmap_width  = max (int (self._pixmap_width  * 1.1), width)
-            create_pixmap = True
+        if DBL_BUFFER:
+            create_pixmap = False
+            if width > self._pixmap_width:
+                # increase the pixmap in 10%+ (rather than 1 pixel) steps
+                self._pixmap_width  = max (int (self._pixmap_width  * 1.1), width)
+                create_pixmap = True
 
-        if height > self._pixmap_height:
-            self._pixmap_height = max (int (self._pixmap_height * 1.1), height)
-            create_pixmap = True
+            if height > self._pixmap_height:
+                self._pixmap_height = max (int (self._pixmap_height * 1.1), height)
+                create_pixmap = True
 
-        if create_pixmap:
-            if DEBUG: print 'FigureCanvasGTK.%s new pixmap' % fn_name()
-            self._pixmap = gtk.gdk.Pixmap (self.window, self._pixmap_width,
-                                           self._pixmap_height)
-            self._renderer._set_pixmap (self._pixmap)
+            if create_pixmap:
+                if DEBUG: print 'FigureCanvasGTK.%s new pixmap' % fn_name()
+                self._pixmap = gtk.gdk.Pixmap (self.window, self._pixmap_width,
+                                               self._pixmap_height)
+                self._renderer._set_pixmap (self._pixmap)
+        else:
+            self._renderer._set_pixmap (self.window)
 
         self._renderer._set_width_height (width, height)
         self.figure.draw (self._renderer)
@@ -249,13 +259,20 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
         Should not be overridden.
         """
         if DEBUG: print 'FigureCanvasGTK.%s' % fn_name()
-        if self._draw_pixmap and GTK_WIDGET_DRAWABLE(self):
-            width, height = self.allocation.width, self.allocation.height
-            self._render_to_pixmap(width, height)
-            self.window.set_back_pixmap (self._pixmap, False)
-            self.window.clear()  # draw pixmap as the gdk.Window's bg
-            self._draw_pixmap = False
-        return True
+
+        if DBL_BUFFER:
+            if self._draw_pixmap and GTK_WIDGET_DRAWABLE(self):
+                width, height = self.allocation.width, self.allocation.height
+                self._render_figure(width, height)
+                self.window.set_back_pixmap (self._pixmap, False)
+                self.window.clear()  # draw pixmap as the gdk.Window's bg
+                self._draw_pixmap = False
+        else:
+            if GTK_WIDGET_DRAWABLE(self):
+                width, height = self.allocation.width, self.allocation.height
+                self._render_figure(width, height)
+            
+        return False # allow signal to propagate further
 
 
     def print_figure(self, filename, dpi=150, facecolor='w', edgecolor='w',
@@ -283,9 +300,15 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
 
         ext = ext.lower()
         if ext in ('jpg', 'png'):          # native printing
+            global DBL_BUFFER
+            dbl_buffer_save = DBL_BUFFER
+            DBL_BUFFER = True  # need to use pixmap not gdk.window
+            
             width, height = self.figure.get_width_height()
             width, height = int(width), int(height)
-            self._render_to_pixmap(width, height)
+            self._render_figure(width, height)
+
+            DBL_BUFFER = dbl_buffer_save
 
             # jpg colors don't match the display very well, png colors match better
             pixbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, 0, 8,
@@ -298,7 +321,7 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
             try:
                 pixbuf.save(filename, ext)
             except gobject.GError, exc:
-                error_msg('Save figure failure:\n%s' % (exc,), parent=self)
+                error_msg_gtk('Save figure failure:\n%s' % (exc,), parent=self)
 
         elif ext in ('eps', 'ps', 'svg',):
             if ext == 'svg':
@@ -310,16 +333,16 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
                 fc = self.switch_backends(FigureCanvas)
                 fc.print_figure(filename, dpi, facecolor, edgecolor, orientation)
             except IOError, exc:
-                error_msg("Save figure failure:\n%s: %s" %
+                error_msg_gtk("Save figure failure:\n%s: %s" %
                           (exc.filename, exc.strerror), parent=self)
             except Exception, exc:
-                error_msg("Save figure failure:\n%s" % exc, parent=self)
+                error_msg_gtk("Save figure failure:\n%s" % exc, parent=self)
 
         elif ext in ('bmp', 'raw', 'rgb',):
             try: 
                 from backend_agg import FigureCanvasAgg  as FigureCanvas
             except:
-                error_msg('Save figure failure:\n'
+                error_msg_gtk('Save figure failure:\n'
                           'Agg must be installed to save as bmp, raw and rgb',
                           parent=self)                
             else:
@@ -330,7 +353,7 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
             try: 
                 from backend_cairo import FigureCanvasCairo  as FigureCanvas
             except:
-                error_msg('Save figure failure:\n'
+                error_msg_gtk('Save figure failure:\n'
                           'Cairo must be installed to save as pdf',
                           parent=self)                
             else:
@@ -338,7 +361,7 @@ class FigureCanvasGTK(gtk.DrawingArea, FigureCanvasBase):
                 fc.print_figure(filename, dpi, facecolor, edgecolor, orientation)
 
         else:
-            error_msg('Format "%s" is not supported.\nSupported formats are %s.' %
+            error_msg_gtk('Format "%s" is not supported.\nSupported formats are %s.' %
                       (ext, ', '.join(IMAGE_FORMAT)),
                       parent=self)
 
@@ -413,6 +436,7 @@ class FigureManagerGTK(FigureManagerBase):
         if Gcf.get_num_fig_managers()==0 and not matplotlib.is_interactive():
             gtk.main_quit()
 
+
         
 class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
     # list of toolitems to add to the toolbar, format is:
@@ -445,13 +469,9 @@ class NavigationToolbar2GTK(NavigationToolbar2, gtk.Toolbar):
         except AttributeError: pass
 
     def dynamic_update(self):
-        def idle_draw(*args):
-            self.canvas.draw()
-            self._idleId = 0
-            return False
-        if self._idleId==0:
-            self._idleId = gtk.idle_add(idle_draw)
-        
+        # legacy method; new method is canvas.draw_idle
+        self.canvas.draw_idle()
+            
     def draw_rubberband(self, event, x0, y0, x1, y1):
         'adapted from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/189744'
         drawable = self.canvas.window
@@ -933,7 +953,7 @@ if gtk.pygtk_version >= (2,4,0):
                     self.path = filename
                     break
                 else:
-                    error_msg('Image format "%s" is not supported' % ext,
+                    error_msg_gtk('Image format "%s" is not supported' % ext,
                               parent=self)
                     self.set_current_name(os.path.split(root)[1] + '.' + menu_ext)
                     
@@ -950,6 +970,7 @@ except:
     verbose.report('Could not load matplotlib icon: %s' % sys.exc_info()[1])
 
 
+
 def error_msg_gtk(msg, parent=None):
 
     if parent: # find the toplevel gtk.Window
@@ -989,4 +1010,6 @@ def exception_handler(type, value, tb):
 #    sys.excepthook = exception_handler
 
 FigureManager = FigureManagerGTK
-error_msg = error_msg_gtk
+
+
+
