@@ -2,7 +2,7 @@ from __future__ import division
 
 from matplotlib import verbose, rcParams, __version__
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
-     FigureManagerBase, FigureCanvasBase, error_msg
+     FigureManagerBase, FigureCanvasBase
 
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.colors import rgb2hex
@@ -67,13 +67,8 @@ class RendererSVG(RendererBase):
         font = _fontd.get(key)
         if font is None:
             fname = fontManager.findfont(prop)
-            try:
-                font = FT2Font(str(fname))
-            except RuntimeError, msg:
-                verbose.report_error('Could not load filename for text "%s"'%fname)
-                return None
-            else:
-                _fontd[key] = font
+            font = FT2Font(str(fname))
+            _fontd[key] = font
         font.clear()
         size = prop.get_size_in_points()
         font.set_size(size, 72.0)
@@ -130,7 +125,8 @@ class RendererSVG(RendererBase):
         """
 
         if len(x)==0: return
-        if len(x)!=len(y): error_msg('x and y must be the same length')
+        if len(x)!=len(y):
+            raise ValueError('x and y must be the same length')
         type = '<path '
 
         y = self.height - y
@@ -170,7 +166,7 @@ class RendererSVG(RendererBase):
         for fontname, fontsize, num, ox, oy, metrics in svg_glyphs:
             thetext=unichr(num)
             thetext.encode('utf-8')
-            style = 'font-size: %f; font-family: %s; stroke-width: 0.5; stroke: %s; fill: %s;'%(fontsize, fontname, color, color)
+            style = 'font-size: %f; font-family: %s; fill: %s;'%(fontsize, fontname, color)
             if angle!=0:
                 transform = 'transform="translate(%f,%f) rotate(%1.1f) translate(%f,%f)"' % (x,y,-angle,-x,-y) # Inkscape doesn't support rotate(angle x y)
             else: transform = ''
@@ -191,23 +187,20 @@ class RendererSVG(RendererBase):
         
         font = self._get_font(prop)
 
-        fontsize = prop.get_size_in_points()
-        
         thetext = '%s' % s
-        fontname = font.get_sfnt()[(1,0,0,6)]
+        fontfamily=font.family_name
+        fontstyle=font.style_name
         fontsize = prop.get_size_in_points()
         color = rgb2hex(gc.get_rgb())
-        style = 'font-size: %f; font-family: %s; stroke-width: 0.5; stroke: %s; fill: %s;'%(fontsize, fontname, color, color)
-        #style = 'font-size: %f; font-family: %s; stroke: %s;'%(fontsize, fontname, color)
+
+        style = 'font-size: %f; font-family: %s; font-style: %s; fill: %s;'%(fontsize, fontfamily,fontstyle, color)
         if angle!=0:
             transform = 'transform="translate(%f,%f) rotate(%1.1f) translate(%f,%f)"' % (x,y,-angle,-x,-y) # Inkscape doesn't support rotate(angle x y)
         else: transform = ''
 
 
         svg = """\
-<text style="%(style)s" x="%(x)f" y="%(y)f" %(transform)s>
-  %(thetext)s
-</text>
+<text style="%(style)s" x="%(x)f" y="%(y)f" %(transform)s>%(thetext)s</text>
 """ % locals()
         self._draw_rawsvg(svg)
 
@@ -284,12 +277,6 @@ style="%(style)s %(rgbhex)s %(clippath)s "
         return GraphicsContextSVG()
 
 class GraphicsContextSVG(GraphicsContextBase):
-
-    def set_linestyle(self, style):
-        GraphicsContextBase.set_linestyle(self, style)
-        offset, dashes = self.dashd[style]
-        self.set_dashes(offset, dashes)
-
     def get_capstyle(self):
         'one of butt/round/square/none'
         d = {'projecting' : 'square',
