@@ -84,6 +84,8 @@ def new_figure_manager(num, *args, **kwargs):
     window = Tk.Tk()
     canvas = FigureCanvasTkAgg(figure, master=window)    
     figManager = FigureManagerTkAgg(canvas, num, window)
+    if matplotlib.is_interactive():
+        figManager.show()
     return figManager
 
 
@@ -98,6 +100,7 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
 
     def __init__(self, figure, master=None, resize_callback=None):
         FigureCanvasAgg.__init__(self, figure)
+        self._idle = True
         t1,t2,w,h = self.figure.bbox.get_bounds()
         w, h = int(w), int(h)
         self._tkcanvas = Tk.Canvas(
@@ -146,6 +149,16 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
 
     show = draw
 
+    def draw_idle(self):
+        'update drawing area only if idle'
+        d = self._idle
+        self._idle = False
+        def idle_draw(*args):
+            self.draw()
+            self._idle = True
+
+        if d: self._tkcanvas.after_idle(idle_draw)
+
     def get_tk_widget(self):
         """returns the Tk widget used to implement FigureCanvasTkAgg.
         Although the initial implementation uses a Tk canvas,  this routine
@@ -165,7 +178,7 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
         x = event.x
         # flipy so y=0 is bottom of canvas
         y = self.figure.bbox.height() - event.y
-        FigureCanvasBase.motion_notify_event(self, x, y)
+        FigureCanvasBase.motion_notify_event(self, x, y, guiEvent=event)
 
     def button_press_event(self, event):
         x = event.x
@@ -179,7 +192,7 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
             if   num==2: num=3
             elif num==3: num=2
 
-        FigureCanvasBase.button_press_event(self, x, y, num)
+        FigureCanvasBase.button_press_event(self, x, y, num, guiEvent=event)
 
     def button_release_event(self, event):
         x = event.x
@@ -194,7 +207,7 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
             if   num==2: num=3
             elif num==3: num=2
 
-        FigureCanvasBase.button_release_event(self, x, y, num)
+        FigureCanvasBase.button_release_event(self, x, y, num, guiEvent=event)
 
     def _get_key(self, event):
         val = event.keysym_num
@@ -209,11 +222,11 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
         
     def key_press(self, event):
         key = self._get_key(event)
-        FigureCanvasBase.key_press_event(self, key)        
+        FigureCanvasBase.key_press_event(self, key, guiEvent=event)        
 
     def key_release(self, event):
         key = self._get_key(event)
-        FigureCanvasBase.key_release_event(self, key)        
+        FigureCanvasBase.key_release_event(self, key, guiEvent=event)        
 
 
     
@@ -275,7 +288,8 @@ class FigureManagerTkAgg(FigureManagerBase):
             self.window.deiconify()
             # anim.py requires this
             if sys.platform=='win32' : self.window.update()            
-        else: self.canvas.draw()
+        else:
+            self.canvas.draw()
         self._shown = True
 
 
@@ -613,14 +627,9 @@ class NavigationToolbar2TkAgg(NavigationToolbar2, Tk.Frame):
 
     def dynamic_update(self):
         'update drawing area only if idle'
-        d = self._idle
-        self._idle = False
-        def idle_draw(*args):
-            self.canvas.draw()
-            self._idle = True
+        # legacy method; new method is canvas.draw_idle
+        self.canvas.draw_idle()
 
-        if d: self.canvas._tkcanvas.after_idle(idle_draw)
-            
 
 FigureManager = FigureManagerTkAgg
-error_msg = error_msg_tkpaint
+
