@@ -10,9 +10,6 @@ from numerix import nx
 LUTSIZE = rcParams['image.lut']
 
 
-_cool_data = {'red':   ((),()),
-              'green': ((),()),
-              'blue':  ((),())}
               
 _bone_data = {'red':   ((0., 0., 0.),(1.0, 1.0, 1.0)),
               'green': ((0., 0., 0.),(1.0, 1.0, 1.0)),
@@ -386,13 +383,6 @@ def get_cmap(name=None, lut=None):
 
 # These are provided for backwards compat
 import sys
-def ColormapJet(N=LUTSIZE):
-    verbose.report_error("ColormapJet deprecated, please use cm.jet instead")
-    return colors.LinearSegmentedColormap('jet',  _jet_data, N)
-
-def Grayscale(N=LUTSIZE):
-    verbose.report_error("Grayscale deprecated, please use cm.gray instead")
-    return colors.LinearSegmentedColormap('gray',  _gray_data, N)
 
 
 class ScalarMappable:    
@@ -431,6 +421,10 @@ class ScalarMappable:
         'Set the image array from numeric/numarray A'
         self._A = A.astype(nx.Float32)
 
+    def get_array(self):
+        'Return the array'
+        return self._A 
+
     def set_clim(self, vmin=None, vmax=None):
         'set the norm limits for image scaling'
         self.norm.vmin = vmin
@@ -471,7 +465,7 @@ class ScalarMappable:
         cmap of another image
         """
         self.observers.append(mappable)
-        
+
     def notify(self, mappable):
         """
         If this is called then we are pegged to another mappable.
