@@ -33,11 +33,13 @@ License   : matplotlib license (PSF compatible)
             see license/LICENSE_TTFQUERY.
 """
 
-import os, sys, glob
+import os, sys, glob, warnings
+import matplotlib
 from matplotlib import afm
 from matplotlib import ft2font
 from matplotlib import rcParams, get_data_path, get_home
 
+verbose = matplotlib.verbose
 
 font_scalings = {'xx-small': 0.579, 'x-small': 0.694, 'small': 0.833,
                  'medium': 1.0, 'large': 1.200, 'x-large': 1.440,
@@ -62,14 +64,19 @@ X11FontDirectories  = [
     # common application, not really useful
     "/usr/lib/openoffice/share/fonts/truetype/",
     # documented as a good place to install new fonts...
-    "/usr/share/fonts",
-    # okay, now the OS X variants...
-    "~/Library/Fonts/",
+    "/usr/share/fonts/"]
+    
+OSXFontDirectories = [
     "/Library/Fonts/",
     "/Network/Library/Fonts/",
-    "/System/Library/Fonts/",
-    "System Folder:Fonts:"]
+    "/System/Library/Fonts/"
+]
 
+home = os.environ.get('HOME')
+if home is not None:
+    # user fonts on OSX
+    path = os.path.join(home, 'Library', 'Fonts')
+    OSXFontDirectories.append(path)
 
 def win32FontDirectory():
     """Return the user-specified font directory for Win32."""
@@ -108,17 +115,52 @@ returned by default with AFM fonts as an option.
             return glob.glob(os.path.join(directory, '*.'+fontext))
         try:
             for j in range(_winreg.QueryInfoKey(local)[1]):
-                key, direc, any = _winreg.EnumValue( local, j)
-                if not os.path.dirname(direc):
-                    direc = os.path.join(directory, direc)
-                direc = os.path.abspath(direc).lower()
-                if direc[-4:] == '.'+fontext:
-                    items[direc] = 1
+                try:
+                    key, direc, any = _winreg.EnumValue( local, j)
+                    if not os.path.dirname(direc):
+                        direc = os.path.join(directory, direc)
+                    direc = os.path.abspath(direc).lower()
+                    if direc[-4:] == '.'+fontext:
+                        items[direc] = 1
+                except EnvironmentError:
+                    continue
+                except WindowsError:
+                    continue
+                        
             return items.keys()
         finally:
             _winreg.CloseKey(local)
     return None
 
+def OSXFontDirectory():
+    """Return the system font directories for OS X."""
+
+    fontpaths = []
+    def add(arg,directory,files):
+        fontpaths.append(directory)
+    for fontdir in OSXFontDirectories:
+        try:
+            if os.path.isdir(fontdir):
+                os.path.walk(fontdir, add, None)
+        except (IOError, OSError, TypeError, ValueError):
+            pass
+    return fontpaths
+
+def OSXInstalledFonts(directory=None, fontext=None):
+    """Get list of font files on OS X - ignores font suffix by default"""
+    if directory is None:
+        directory = OSXFontDirectory()
+    
+    files = []
+    for path in directory:
+        if fontext is None:
+            files.extend(glob.glob(os.path.join(path,'*')))
+        else:
+            files.extend(glob.glob(os.path.join(path, '*.'+fontext)))
+            files.extend(glob.glob(os.path.join(path, '*.'+fontext.upper())))
+    return files
+
+
 def x11FontDirectory():
     """Return the system font directories for X11."""
 
@@ -130,12 +172,12 @@ def x11FontDirectory():
     else:
         fontpaths = []
         #def add(arg, directory, files):
-        def add(directory):
+        def add(arg,directory,files):
             fontpaths.append(directory)
         for fontdir in X11FontDirectories:
             try:
                 if os.path.isdir(fontdir):
-                    os.path.walk(fontdir, add, ())
+                    os.path.walk(fontdir, add, None)
             except (IOError, OSError, TypeError, ValueError):
                 pass
         return fontpaths
@@ -162,10 +204,17 @@ with AFM fonts as an option.
                     fontfiles[f] = 1
         else:
             fontpaths = x11FontDirectory()
+            # check for OS X & load its fonts if present
+            if sys.platform == 'darwin':
+                for f in OSXInstalledFonts():
+                    fontfiles[f] = 1
+
     elif isinstance(fontpaths, (str, unicode)):
         fontpaths = [fontpaths]
     for path in fontpaths:
-        for fname in glob.glob(os.path.join(path, '*.'+fontext)):
+        files = glob.glob(os.path.join(path, '*.'+fontext))
+        files.extend(glob.glob(os.path.join(path, '*.'+fontext.upper())))
+        for fname in files:
 
             fontfiles[os.path.abspath(fname)] = 1
     return [fname for fname in fontfiles.keys() if os.path.exists(fname)]
@@ -402,21 +451,20 @@ dictionary can optionally be created.
         fname = fpath.split('/')[-1]
         if seen.has_key(fname):  continue
         else: seen[fname] = 1
-        if fontext == 'ttf':
+        if fontext == 'afm':
+            try:
+                font = afm.AFM(file(fpath))
+            except RuntimeError:
+                warnings.warn("Could not open font file %s"%fpath)
+                continue
+            prop = afmFontProperty(font)
+        else:
             try:
                 font = ft2font.FT2Font(str(fpath))
             except RuntimeError:
-                print >> sys.stderr, "Could not open font file", fpath
+                warnings.warn("Could not open font file %s"%fpath)
                 continue
             prop = ttfFontProperty(font)
-        elif fontext == 'afm':
-            try:
-                #print 'parsing', fpath
-                font = afm.AFM(file(fpath))
-            except RuntimeError:
-                print >> sys.stderr, "Could not open font file", fpath
-                continue
-            prop = afmFontProperty(font)
         add_filename(fontdict, prop, fpath)
 
         #  !!!!  Default font algorithm needs improvement
@@ -547,6 +595,7 @@ Examples:
                  weight = None,
                  stretch= None,
                  size   = None,
+                 fname = None, # if this is set, it's a hardcoded filename to use
                  ):
 
 
@@ -566,12 +615,13 @@ Examples:
         self.__stretch = stretch
         self.__size    = size
         self.__parent_size = fontManager.get_default_size()
-
+        self.fname = fname
+        
     def __hash__(self):
         return hash( (
             tuple(self.__family), self.__style, self.__variant,
             self.__weight, self.__stretch, self.__size,
-            self.__parent_size))
+            self.__parent_size, self.fname))
 
     def __str__(self):
         return str((self.__family, self.__style, self.__variant,
@@ -644,7 +694,7 @@ set_family() method.
         #  Remove family names from list of font names.
         for name in names[:]:
             if name.lower() in font_family:
-                print >> sys.stderr, msg % name
+                warnings.warn( msg % name)
                 while name in names:
                     names.remove(name.lower())
 
@@ -747,10 +797,13 @@ font dictionary can act like a font cache.
                 else:
                     paths.append(ttfpath)
 
+        verbose.report('font search path %s'%(str(paths)))
         #  Load TrueType fonts and create font dictionary.
         
         self.ttffiles = findSystemFonts(paths) + findSystemFonts()
+
         for fname in self.ttffiles:
+            verbose.report('trying fontname %s' % fname, 'debug')
             if fname.lower().find('vera.ttf')>=0:
                 self.defaultFont = fname
                 break
@@ -774,10 +827,10 @@ Delete this file to have matplotlib rebuild the cache."""
         def rebuild():
             self.ttfdict = createFontDict(self.ttffiles)
             pickle.dump(self.ttfdict, file(ttfcache, 'w'))
-            print cache_message % ttfcache
+            verbose.report(cache_message % ttfcache)
             
         try:
-            self.ttfdict = pickle.load(file(ttfcache))            
+            self.ttfdict = pickle.load(file(ttfcache))
         except:
             rebuild()
         else:
@@ -786,7 +839,7 @@ Delete this file to have matplotlib rebuild the cache."""
                 if not os.path.exists(fname):
                     rebuild()
                     break
-
+            verbose.report('loaded ttfcache file %s'%ttfcache)
             
 
 
@@ -822,6 +875,7 @@ Currently not implemented."""
         #  !!!!  Needs implementing
         raise NotImplementedError
 
+    
     def findfont(self, prop, fontext='ttf'):
 
         """Search the font dictionary for a font that exactly or closely
@@ -841,10 +895,12 @@ documentation for a description of the font finding algorithm.
 Delete this file to have matplotlib rebuild the cache."""
 
         debug = False
-
-        if   fontext == 'ttf':
-            fontdict = self.ttfdict
-        elif fontext == 'afm':
+        if prop.fname is not None:
+            fname = prop.fname
+            verbose.report('findfont returning %s'%fname, 'debug')
+            return fname
+        
+        if fontext == 'afm':
             if len(self.afmdict) == 0:
                 afmpath = os.environ.get('HOME', get_data_path())
                 afmcache = os.path.join(afmpath, '.afmfont.cache')
@@ -857,8 +913,10 @@ Delete this file to have matplotlib rebuild the cache."""
                 except:
                     self.afmdict = createFontDict(self.afmfiles, fontext='afm')
                     pickle.dump(self.afmdict, file(afmcache, 'w'))
-                    print cache_message % afmcache
+                    verbose.report(cache_message % afmcache)
             fontdict = self.afmdict
+        else:
+            fontdict = self.ttfdict
 
         name    = prop.get_family()[0]
         style   = prop.get_style()
@@ -867,10 +925,11 @@ Delete this file to have matplotlib rebuild the cache."""
         stretch = prop.get_stretch()
         size    = str(prop.get_size_in_points())
 
+
         try:
             fname = fontdict[name][style][variant][weight][stretch][size]
-            if debug:
-                print 'Cache:', name, style, variant, weight, stretch, size
+            verbose.report('\tfindfont cached %(name)s, %(style)s, %(variant)s, %(weight)s, %(stretch)s, %(size)s'%locals(), 'debug')
+            verbose.report('findfont returning %s'%fname, 'debug')
             return fname
         except KeyError:
             pass
@@ -880,24 +939,21 @@ Delete this file to have matplotlib rebuild the cache."""
             if font.has_key(name):
                 font = font[name]
             else:
-                if debug:
-                    print name
+                verbose.report('\tfindfont failed %(name)s'%locals(), 'debug')
                 continue
-
+                
             if font.has_key(style):
                 font = font[style]
             elif style == 'italics' and font.has_key('oblique'):
                 font = font['oblique']
             else:
-                if debug:
-                    print name, style
+                verbose.report('\tfindfont failed %(name)s, %(style)s'%locals(), 'debug')
                 continue
 
             if font.has_key(variant):
                 font = font[variant]
             else:
-                if debug:
-                    print name, style, variant
+                verbose.report('\tfindfont failed %(name)s, %(style)s, %(variant)s'%locals(), 'debug')
                 continue
 
             if not font.has_key(weight):
@@ -908,8 +964,7 @@ Delete this file to have matplotlib rebuild the cache."""
             if font.has_key(stretch):
                 font = font[stretch]
             else:
-                if debug:
-                    print name, style, variant, weight, stretch
+                verbose.report('\tfindfont failed %(name)s, %(style)s, %(variant)s %(weight)s, %(stretch)s'%locals(), 'debug')
                 continue
 
             if font.has_key('scalable'):
@@ -917,27 +972,20 @@ Delete this file to have matplotlib rebuild the cache."""
             elif font.has_key(size):
                 fname = font[size]
             else:
-                if debug:
-                    print name, style, variant, weight, stretch, size
+                verbose.report('\tfindfont failed %(name)s, %(style)s, %(variant)s %(weight)s, %(stretch)s, %(size)s'%locals(), 'debug')                
                 continue
 
             fontkey = FontKey(name, style, variant, weight, stretch, size)
             add_filename(fontdict, fontkey, fname)
-            if debug:
-                print 'Found:', name, style, variant, weight, stretch, size
-            
+            verbose.report('\tfindfont found %(name)s, %(style)s, %(variant)s %(weight)s, %(stretch)s, %(size)s'%locals(), 'debug')
+            verbose.report('findfont returning %s'%fname, 'debug')
+                
             return fname
 
-        if debug:
-            print name, style, variant, weight, stretch, size
         fontkey = FontKey(name, style, variant, weight, stretch, size)
         add_filename(fontdict, fontkey, self.defaultFont)
-        print >> sys.stderr, 'Could not match %s, %s, %s.  Returning %s' % (name, style, variant, self.defaultFont)
+        warnings.warn('Could not match %s, %s, %s.  Returning %s' % (name, style, variant, self.defaultFont))
 
         return self.defaultFont
-        #raise KeyError, """Cannot match font family with known fonts.
-        #Please set the environment variable TTFPATH or AFMPATH to your fonts.
-        #"""+str(prop.get_family())
-
 
 fontManager = FontManager()
