@@ -5,7 +5,7 @@ specifying the legend location
 
 The location codes are
 
-  'best'         : 0,  (currently not supported, defaults to upper right)
+  'best'         : 0,  
   'upper right'  : 1,  (default)
   'upper left'   : 2,
   'lower left'   : 3,
@@ -45,7 +45,7 @@ class Legend(Artist):
 
     The location codes are
 
-      'best'         : 0,  (currently not supported, defaults to upper right)
+      'best'         : 0, 
       'upper right'  : 1,  (default)
       'upper left'   : 2,
       'lower left'   : 3,
@@ -99,7 +99,7 @@ class Legend(Artist):
   loc                   # a location code
   isaxes=True           # whether this is an axes legend
   numpoints = 4         # the number of points in the legend line
-  fontprop = FontProperties('smaller')  # the font property
+  fontprop = FontProperties(size='smaller')  # the font property
   pad = 0.2             # the fractional whitespace inside the legend border
   markerscale = 0.6     # the relative size of legend markers vs. original
   shadow                # if True, draw a shadow behind legend 
@@ -113,7 +113,7 @@ The following dimensions are in axes coords
         Artist.__init__(self)
         if is_string_like(loc) and not self.codes.has_key(loc):
             warnings.warn('Unrecognized location %s. Falling back on upper right; valid locations are\n%s\t' %(loc, '\n\t'.join(self.codes.keys())))
-        if is_string_like(loc): loc = self.codes.get(loc, 0)
+        if is_string_like(loc): loc = self.codes.get(loc, 1)
         
         self.numpoints = numpoints
         self.prop = prop
