@@ -6,8 +6,9 @@ variety of line styles, markers and colors
 # TODO: expose cap and join style attrs
 from __future__ import division
 
-import sys, math
+import sys, math, warnings
 
+import agg
 from numerix import Float, alltrue, arange, array, logical_and,\
      nonzero, searchsorted, take, asarray, ones, where, less, ravel, \
      greater, logical_and, cos, sin, pi
@@ -16,7 +17,7 @@ from artist import Artist
 from cbook import iterable, is_string_like
 from colors import colorConverter
 from patches import bbox_artist
-from path import MOVETO, LINETO, ENDPOLY
+
 from transforms import lbwh_to_bbox, LOG10
 from matplotlib import rcParams
 
@@ -134,13 +135,11 @@ class Line2D(Artist):
         self.set_data(xdata, ydata)
 
         if not self._lineStyles.has_key(linestyle):
-            verbose.report_error('Unrecognized line style %s, %s' %( linestyle, type(linestyle)))
+            raise ValueError('Unrecognized line style %s, %s' %( linestyle, type(linestyle)))
         if not self._markers.has_key(marker):
-            verbose.report_error('Unrecognized marker style %s, %s'%( marker, type(marker)))
+            raise ValueError('Unrecognized marker style %s, %s'%( marker, type(marker)))
 
         self.set_marker(marker)
-        self._lineFunc = self._lineStyles.get(linestyle, self._draw_nothing)
-        self._markerFunc = self._markers.get(marker, self._draw_nothing)
 
         self._logcache = None
             
@@ -190,8 +189,10 @@ ACCEPTS: (array xdata, array ydata)
         self._x = asarray(x, Float)
         self._y = asarray(y, Float)
 
-        if len(self._x.shape)>1: self._x = ravel(self._x)
-        if len(self._y.shape)>1: self._y = ravel(self._y)
+        if len(self._x.shape)>1:
+            self._x = ravel(self._x)
+        if len(self._y.shape)>1:
+            self._y = ravel(self._y)
 
 
         if len(self._y)==1 and len(self._x)>1:
@@ -259,7 +260,7 @@ ACCEPTS: [True | False]
 
     def draw(self, renderer):
         #renderer.open_group('line2d')
-
+        if not self._visible: return 
         self._newstyle = hasattr(renderer, 'draw_markers')
         gc = renderer.new_gc()
         gc.set_foreground(self._color)
@@ -280,7 +281,8 @@ ACCEPTS: [True | False]
             xt, yt = self._transform.numerix_x_y(x, y)
 
 
-        lineFunc = getattr(self, self._lineFunc)
+        funcname = self._lineStyles.get(self._linestyle, '_draw_nothing')
+        lineFunc = getattr(self, funcname)
         lineFunc(renderer, gc, xt, yt)
 
 
@@ -290,7 +292,8 @@ ACCEPTS: [True | False]
             gc.set_linewidth(self._markeredgewidth)
             if self.get_clip_on():
                 gc.set_clip_rectangle(self.clipbox.get_bounds())
-            markerFunc = getattr(self, self._markerFunc)
+            funcname = self._markers.get(self._marker, '_draw_nothing')
+            markerFunc = getattr(self, funcname)
             markerFunc(renderer, gc, xt, yt)
 
         #if 1: bbox_artist(self, renderer)
@@ -395,7 +398,7 @@ Set the linestyle of the line
 ACCEPTS: [ '-' | '--' | '-.' | ':' | 'steps' | 'None' ]
         """
         self._linestyle = s
-        self._lineFunc = self._lineStyles.get(self._linestyle, '-')
+
 
     def set_marker(self, marker):
         """
@@ -405,7 +408,6 @@ ACCEPTS: [ '+' | ',' | '.' | '1' | '2' | '3' | '4' | '<' | '>' | 'D' | 'H' | '^'
 
 """
         self._marker = marker
-        self._markerFunc = self._markers.get(marker, self._draw_nothing)
 
     def set_markeredgecolor(self, ec):
         """
@@ -497,7 +499,11 @@ Set the dash sequence, sequence of dashes with on off ink in points
 
 ACCEPTS: sequence of on/off ink in points
 """
-        self._dashSeq = seq
+        if seq == (None, None):
+            self.set_linestyle('-')
+        else:
+            self.set_linestyle('--')
+        self._dashSeq = seq  # TODO: offset ignored for now
         
     def _draw_nothing(self, renderer, gc, xt, yt):
         pass
@@ -568,15 +574,12 @@ ACCEPTS: sequence of on/off ink in points
     def _draw_point(self, renderer, gc, xt, yt):
         if self._newstyle:
             rgbFace = self._get_rgb_face()
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, -0.5, -0.5),
-                (LINETO, -0.5, 0.5),
-                (LINETO, 0.5, 0.5),
-                (LINETO, 0.5, -0.5),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(-0.5, -0.5)
+            path.line_to(-0.5, 0.5)
+            path.line_to(0.5, 0.5)
+            path.line_to(0.5, -0.5)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_arc(gc, None, x, y, 1, 1, 0.0, 360.0)
@@ -584,15 +587,12 @@ ACCEPTS: sequence of on/off ink in points
     def _draw_pixel(self, renderer, gc, xt, yt):
         if self._newstyle:
             rgbFace = self._get_rgb_face()            
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, -0.5, -0.5),
-                (LINETO, -0.5, 0.5),
-                (LINETO, 0.5, 0.5),
-                (LINETO, 0.5, -0.5),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(-0.5, -0.5)
+            path.line_to(-0.5, 0.5)
+            path.line_to(0.5, 0.5)
+            path.line_to(0.5, -0.5)
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_point(gc, x, y)
@@ -610,11 +610,14 @@ ACCEPTS: sequence of on/off ink in points
             rads = (2*math.pi/N)*arange(N)
             xs = r*cos(rads)
             ys = r*sin(rads)
-            verts = [(MOVETO, xs[0], ys[0])]
-            verts.extend([(LINETO, x, y) for x, y in zip(xs[1:], ys[1:])])
-            CLOSE = self._get_close(rgbFace)
-            verts.append(CLOSE)
-            renderer.draw_markers(gc, verts, xt, yt, self._transform)
+            # todo: use curve3!
+            path = agg.path_storage()
+            path.move_to(xs[0], ys[0])
+            for x, y in zip(xs[1:], ys[1:]):                
+                path.line_to(x, y) 
+            
+            path.end_poly()
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_arc(gc, rgbFace,
@@ -629,14 +632,12 @@ ACCEPTS: sequence of on/off ink in points
         rgbFace = self._get_rgb_face()
 
         if self._newstyle:
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, 0, offset),
-                (LINETO, -offset, -offset),
-                (LINETO, offset, -offset),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(0, offset)
+            path.line_to(-offset, -offset)
+            path.line_to(offset, -offset)
+            path.end_poly()
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 verts = ( (x, y+offset),
@@ -650,14 +651,14 @@ ACCEPTS: sequence of on/off ink in points
         rgbFace = self._get_rgb_face()
 
         if self._newstyle:
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, -offset, offset),
-                (LINETO, offset, offset),
-                (LINETO, 0, -offset),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            
+            path = agg.path_storage()
+            path.move_to(-offset, offset)
+            path.line_to(offset, offset)
+            path.line_to(0, -offset)
+            path.end_poly()
+                
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):            
                 verts = ( (x-offset, y+offset),
@@ -670,14 +671,14 @@ ACCEPTS: sequence of on/off ink in points
         rgbFace = self._get_rgb_face()
 
         if self._newstyle:
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, -offset, 0),
-                (LINETO, offset, -offset),
-                (LINETO, offset, offset),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            
+            path = agg.path_storage()
+            path.move_to(-offset, 0)
+            path.line_to(offset, -offset)
+            path.line_to(offset, offset)
+            path.end_poly()
+                
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):            
                 verts = ( (x-offset, y),
@@ -690,14 +691,12 @@ ACCEPTS: sequence of on/off ink in points
         offset = 0.5*renderer.points_to_pixels(self._markersize)
         rgbFace = self._get_rgb_face()
         if self._newstyle:
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, offset, 0),
-                (LINETO, -offset, -offset),
-                (LINETO, -offset, offset),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(offset, 0)
+            path.line_to(-offset, -offset)
+            path.line_to(-offset, offset)
+            path.end_poly()
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):            
                 verts = ( (x+offset, y),
@@ -706,13 +705,6 @@ ACCEPTS: sequence of on/off ink in points
                 renderer.draw_polygon(gc, rgbFace, verts)
 
 
-    def _get_close(self, rgbFace):
-        if rgbFace is None:
-            CLOSE = ENDPOLY, 0
-        else:
-            r,g,b = rgbFace
-            CLOSE = ENDPOLY, 1, r, g, b, self._alpha
-        return CLOSE
 
     def _draw_square(self, renderer, gc, xt, yt):
         side = renderer.points_to_pixels(self._markersize)
@@ -720,15 +712,15 @@ ACCEPTS: sequence of on/off ink in points
         rgbFace = self._get_rgb_face()
         
         if self._newstyle:
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, -offset, -offset),
-                (LINETO, -offset, offset),
-                (LINETO, offset, offset),
-                (LINETO, offset, -offset),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            
+            path = agg.path_storage()
+            path.move_to(-offset, -offset)
+            path.line_to(-offset, offset)
+            path.line_to(offset, offset)
+            path.line_to(offset, -offset)
+            path.end_poly()
+                
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
 
             for (x,y) in zip(xt, yt):            
@@ -739,16 +731,15 @@ ACCEPTS: sequence of on/off ink in points
     def _draw_diamond(self, renderer, gc, xt, yt):
         offset = 0.6*renderer.points_to_pixels(self._markersize)
         rgbFace = self._get_rgb_face()
-        if self._newstyle:
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, offset, 0),
-                (LINETO, 0, -offset),
-                (LINETO, -offset, 0),
-                (LINETO, 0, offset),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+        if self._newstyle:            
+            path = agg.path_storage()
+            path.move_to(offset, 0)
+            path.line_to(0, -offset)
+            path.line_to(-offset, 0)
+            path.line_to(0, offset)
+            path.end_poly()
+                
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
 
 
@@ -764,16 +755,14 @@ ACCEPTS: sequence of on/off ink in points
         xoffset = 0.6*offset
         rgbFace = self._get_rgb_face()
 
-        if self._newstyle:
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, xoffset, 0),
-                (LINETO, 0, -offset),
-                (LINETO, -xoffset, 0),
-                (LINETO, 0, offset),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+        if self._newstyle:            
+            path = agg.path_storage()
+            path.move_to(xoffset, 0)
+            path.line_to(0, -offset)
+            path.line_to(-xoffset, 0)
+            path.line_to(0, offset)
+            path.end_poly()                
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):            
                 verts = ( (x+xoffset, y),
@@ -790,17 +779,16 @@ ACCEPTS: sequence of on/off ink in points
         offsetY2 = offset*0.81
         rgbFace = self._get_rgb_face()
 
-        if self._newstyle:
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, 0, offset),
-                (LINETO, -offsetX1, offsetY1),
-                (LINETO, -offsetX2, -offsetY2),
-                (LINETO, +offsetX2, -offsetY2),
-                (LINETO, +offsetX1, offsetY1),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+        if self._newstyle:            
+            path = agg.path_storage()
+            path.move_to(0, offset)
+            path.line_to(-offsetX1, offsetY1)
+            path.line_to(-offsetX2, -offsetY2)
+            path.line_to(+offsetX2, -offsetY2)
+            path.line_to(+offsetX1, offsetY1)
+            path.end_poly()
+                
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):            
                 verts = ( (x, y+offset),
@@ -817,17 +805,15 @@ ACCEPTS: sequence of on/off ink in points
         rgbFace = self._get_rgb_face()
 
         if self._newstyle:
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, 0, offset),
-                (LINETO, -offsetX1, offsetY1),
-                (LINETO, -offsetX1, -offsetY1),
-                (LINETO, 0, -offset),
-                (LINETO, offsetX1, -offsetY1),
-                (LINETO, offsetX1, offsetY1),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(0, offset)
+            path.line_to(-offsetX1, offsetY1)
+            path.line_to(-offsetX1, -offsetY1)
+            path.line_to(0, -offset)
+            path.line_to(offsetX1, -offsetY1)
+            path.line_to(offsetX1, offsetY1)
+            path.end_poly()                
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):            
                 verts = ( (x, y+offset),
@@ -843,21 +829,18 @@ ACCEPTS: sequence of on/off ink in points
         offsetX1 = offset*0.5
         offsetY1 = offset*0.87
         rgbFace = self._get_rgb_face()
-        if self._newstyle:
-            CLOSE = self._get_close(rgbFace)
-            path = (
-                (MOVETO, offset, 0),
-                (LINETO, offsetX1, offsetY1),
-                (LINETO, -offsetX1, offsetY1),
-                (LINETO, -offset, 0),
-                (LINETO, -offsetX1, -offsetY1),
-                (LINETO, offsetX1, -offsetY1),
-                CLOSE
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+        if self._newstyle:            
+            path = agg.path_storage()
+            path.move_to(offset, 0)
+            path.line_to(offsetX1, offsetY1)
+            path.line_to(-offsetX1, offsetY1)
+            path.line_to(-offset, 0)
+            path.line_to(-offsetX1, -offsetY1)
+            path.line_to(offsetX1, -offsetY1)
+            path.end_poly()
+                
+            renderer.draw_markers(gc, path, rgbFace, xt, yt, self._transform)
         else:
-
-
             for (x,y) in zip(xt, yt):            
                 verts = ( (x+offset, y),
                           (x+offsetX1, y+offsetY1),
@@ -870,11 +853,10 @@ ACCEPTS: sequence of on/off ink in points
     def _draw_vline(self, renderer, gc, xt, yt):
         offset = 0.5*renderer.points_to_pixels(self._markersize)
         if self._newstyle:
-            path = (
-                (MOVETO, 0, -offset),
-                (LINETO, 0, offset),
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(0, -offset)
+            path.line_to(0, offset)                
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):            
                 renderer.draw_line(gc, x, y-offset, x, y+offset)
@@ -882,11 +864,10 @@ ACCEPTS: sequence of on/off ink in points
     def _draw_hline(self, renderer, gc, xt, yt):
         offset = 0.5*renderer.points_to_pixels(self._markersize)
         if self._newstyle:
-            path = (
-                (MOVETO, -offset, 0),
-                (LINETO, offset, 0),
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(-offset, 0)
+            path.line_to(offset, 0)                
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x-offset, y, x+offset, y)
@@ -894,11 +875,10 @@ ACCEPTS: sequence of on/off ink in points
     def _draw_tickleft(self, renderer, gc, xt, yt):
         offset = renderer.points_to_pixels(self._markersize)
         if self._newstyle:
-            path = (
-                (MOVETO, -offset, 0),
-                (LINETO, 0, 0),
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(-offset, 0)
+            path.line_to(0, 0)                            
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x-offset, y, x, y)
@@ -907,11 +887,10 @@ ACCEPTS: sequence of on/off ink in points
 
         offset = renderer.points_to_pixels(self._markersize)
         if self._newstyle:
-            path = (
-                (MOVETO, 0, 0),
-                (LINETO, offset, 0),
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(0, 0)
+            path.line_to(offset, 0)                
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x+offset, y)
@@ -919,11 +898,10 @@ ACCEPTS: sequence of on/off ink in points
     def _draw_tickup(self, renderer, gc, xt, yt):
         offset = renderer.points_to_pixels(self._markersize)
         if self._newstyle:
-            path = (
-                (MOVETO, 0, 0),
-                (LINETO, 0, offset),
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(0, 0)
+            path.line_to(0, offset)                
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x, y+offset)
@@ -931,11 +909,10 @@ ACCEPTS: sequence of on/off ink in points
     def _draw_tickdown(self, renderer, gc, xt, yt):
         offset = renderer.points_to_pixels(self._markersize)
         if self._newstyle:
-            path = (
-                (MOVETO, 0, -offset),
-                (LINETO, 0, 0),
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(0, -offset)
+            path.line_to(0, 0)                
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y-offset, x, y)
@@ -943,13 +920,13 @@ ACCEPTS: sequence of on/off ink in points
     def _draw_plus(self, renderer, gc, xt, yt):
         offset = 0.5*renderer.points_to_pixels(self._markersize)
         if self._newstyle:
-            path = (
-                (MOVETO, -offset, 0),
-                (LINETO, offset, 0),
-                (MOVETO, 0, -offset),
-                (LINETO, 0, offset),
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            
+            path = agg.path_storage()
+            path.move_to(-offset, 0)
+            path.line_to( offset, 0)
+            path.move_to( 0, -offset)
+            path.line_to( 0, offset)
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x-offset, y, x+offset, y)
@@ -960,15 +937,14 @@ ACCEPTS: sequence of on/off ink in points
         offset1 = offset*0.8
         offset2 = offset*0.5
         if self._newstyle:
-            path = (
-                (MOVETO, 0, 0),
-                (LINETO, 0, -offset),
-                (MOVETO, 0, 0),
-                (LINETO, offset1, offset2),
-                (MOVETO, 0, 0),
-                (LINETO, -offset1, offset2),
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(0, 0)
+            path.line_to(0, -offset)
+            path.move_to(0, 0)
+            path.line_to(offset1, offset2)
+            path.move_to(0, 0)
+            path.line_to(-offset1, offset2)                
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x, y-offset)
@@ -980,15 +956,14 @@ ACCEPTS: sequence of on/off ink in points
         offset1 = offset*0.8
         offset2 = offset*0.5
         if self._newstyle:
-            path = (
-                (MOVETO, 0, 0),
-                (LINETO, 0, offset),
-                (MOVETO, 0, 0),
-                (LINETO, offset1, -offset2),
-                (MOVETO, 0, 0),
-                (LINETO, -offset1, -offset2),
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(0, 0)
+            path.line_to(0, offset)
+            path.move_to(0, 0)
+            path.line_to(offset1, -offset2)
+            path.move_to(0, 0)
+            path.line_to(-offset1, -offset2)                
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x, y+offset)
@@ -1000,15 +975,14 @@ ACCEPTS: sequence of on/off ink in points
         offset1 = offset*0.8
         offset2 = offset*0.5
         if self._newstyle:
-            path = (
-                (MOVETO, 0, 0),
-                (LINETO, -offset, 0),
-                (MOVETO, 0, 0),
-                (LINETO, offset2, offset1),
-                (MOVETO, 0, 0),
-                (LINETO, offset2, -offset1),
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(0, 0)
+            path.line_to(-offset, 0)
+            path.move_to(0, 0)
+            path.line_to(offset2, offset1)
+            path.move_to(0, 0)
+            path.line_to(offset2, -offset1)                
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x-offset, y)
@@ -1020,15 +994,14 @@ ACCEPTS: sequence of on/off ink in points
         offset1 = offset*0.8
         offset2 = offset*0.5
         if self._newstyle:
-            path = (
-                (MOVETO, 0, 0),
-                (LINETO, offset, 0),
-                (MOVETO, 0, 0),
-                (LINETO, -offset2, offset1),
-                (MOVETO, 0, 0),
-                (LINETO, -offset2, -offset1),
-                )
-            renderer.draw_markers(gc, path, xt, yt, self._transform)
+            path = agg.path_storage()
+            path.move_to(0, 0)
+            path.line_to(offset, 0)
+            path.move_to(0, 0)
+            path.line_to(-offset2, offset1)
+            path.move_to(0, 0)
+            path.line_to(-offset2, -offset1)                
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
         else:
             for (x,y) in zip(xt, yt):
                 renderer.draw_line(gc, x, y, x+offset, y)
@@ -1037,9 +1010,18 @@ ACCEPTS: sequence of on/off ink in points
 
     def _draw_x(self, renderer, gc, xt, yt):
         offset = 0.5*renderer.points_to_pixels(self._markersize)
-        for (x,y) in zip(xt, yt):
-            renderer.draw_line(gc, x-offset, y-offset, x+offset, y+offset)
-            renderer.draw_line(gc, x-offset, y+offset, x+offset, y-offset)
+
+        if self._newstyle:
+            path = agg.path_storage()
+            path.move_to(-offset, -offset)
+            path.line_to(offset, offset)
+            path.move_to(-offset, offset)
+            path.line_to(offset, -offset)                
+            renderer.draw_markers(gc, path, None, xt, yt, self._transform)
+        else:
+            for (x,y) in zip(xt, yt):
+                renderer.draw_line(gc, x-offset, y-offset, x+offset, y+offset)
+                renderer.draw_line(gc, x-offset, y+offset, x+offset, y-offset)
 
     def update_from(self, other):
         'copy properties from other to self'
@@ -1055,8 +1037,6 @@ ACCEPTS: sequence of on/off ink in points
 
         self._linestyle = other._linestyle
         self._marker = other._marker
-        self._lineFunc = other._lineStyles[other._linestyle]
-        self._markerFunc = other._markers[other._marker]
         self._useDataClipping = other._useDataClipping
 
     def _get_rgb_face(self):
