@@ -26,6 +26,8 @@ matplotlib.
 The basic generic  locators are
 
   * NullLocator     - No ticks
+  
+  * FixedLocator    - Tick locations are fixed
 
   * IndexLocator    - locator for index plots (eg where x = range(len(y))
   
@@ -73,8 +75,6 @@ axis.
   * FuncFormatter      - user defined function sets the labels
 
   * FormatStrFormatter - use a sprintf format string
-
-  * IndexFormatter     - cycle through fixed strings by tick position
   
   * ScalarFormatter    - default formatter for scalars; autopick the fmt string
 
@@ -108,7 +108,7 @@ multiple axes can share the same locator w/o side effects!
 
 
 from __future__ import division
-import sys, os, re, time, math
+import sys, os, re, time, math, warnings
 from mlab import linspace
 from matplotlib import verbose
 from numerix import arange, array, asarray, ones, zeros, \
@@ -212,7 +212,7 @@ class ScalarFormatter(Formatter):
         #if the number is not too big and it's an int, format it as an
         #int
         if abs(x)<1e4 and x==int(x): return '%d' % x
- 
+
         if d < 1e-2: fmt = '%1.3e'
         elif d < 1e-1: fmt = '%1.3f'
         elif d > 1e5: fmt = '%1.1e'
@@ -220,7 +220,7 @@ class ScalarFormatter(Formatter):
         elif d > 1 : fmt = '%1.2f'
         else: fmt = '%1.3f'
         s =  fmt % x
-
+        #print d, x, fmt, s
         tup = s.split('e')
         if len(tup)==2:
             mantissa = tup[0].rstrip('0').rstrip('.')
@@ -495,13 +495,13 @@ class Base:
     def lt(self, x):
         'return the largest multiple of base < x'
         d,m = divmod(x, self._base)
-        if m==0: return (d-1)*self._base
+        if closeto(m,0) and not closeto(m/self._base,1): return (d-1)*self._base
         else: return d*self._base
 
     def le(self, x):
         'return the largest multiple of base <= x'
         d,m = divmod(x, self._base)
-        if closeto(m, self._base):
+        if closeto(m/self._base,1): # was closeto(m, self._base)
             #looks like floating point error
             return (d+1)*self._base
         else:
@@ -510,20 +510,17 @@ class Base:
     def gt(self, x):
         'return the largest multiple of base > x'
         d,m = divmod(x, self._base)
-        if closeto(m, self._base):
+        if closeto(m/self._base,1):
             #looks like floating point error
             return (d+2)*self._base
         else:
             return (d+1)*self._base
 
-
-
     def ge(self, x):
         'return the largest multiple of base >= x'
         d,m = divmod(x, self._base)
-        if m==0: return x
+        if closeto(m,0) and not closeto(m/self._base,1): return x
         return (d+1)*self._base
-
     def get_base(self):
         return self._base
     
@@ -705,7 +702,7 @@ class AutoLocator(Locator):
 
             try: ld = math.log10(d)
             except OverflowError:
-                verbose.report_error('AutoLocator illegal dataInterval range %s; returning NullLocator'%d)
+                warnings.warn('AutoLocator illegal dataInterval range %s; returning NullLocator'%d)
                 return NullLocator()
 
             fld = math.floor(ld)
