@@ -37,6 +37,8 @@ basedir = {
     'linux'  : ['/usr/local', '/usr',],
     'darwin' : ['/usr/local', '/usr', '/sw', '/usr/X11R6'],
     'freebsd4' : ['/usr/local', '/usr'],
+    'freebsd5' : ['/usr/local', '/usr'],
+    'freebsd6' : ['/usr/local', '/usr'],    
     'sunos5' : [os.getenv('MPLIB_BASE') or '/usr/local',],
 }
 
@@ -59,6 +61,10 @@ BUILT_IMAGE     = False
 BUILT_TKAGG     = False
 BUILT_WINDOWING = False
 BUILT_CONTOUR   = False
+BUILT_ENTHOUGHT   = False
+BUILT_CONTOUR   = False
+BUILT_GDK       = False
+
 
 class CleanUpFile:
     """CleanUpFile deletes the specified filename when self is destroyed."""
@@ -296,10 +302,10 @@ def add_tk_flags(module):
             module.include_dirs.extend(tk_include_dirs)
             module.extra_link_args.extend(frameworks)
             module.extra_compile_args.extend(frameworks)
-            return
+            return 
 
     # you're still here? ok we'll try it this way
-    o = find_tcltk()
+    o = find_tcltk() # todo: try/except
     module.include_dirs.extend([o.tcl_inc, o.tk_inc])
     module.library_dirs.extend([o.tcl_lib, o.tk_lib])
     module.libraries.extend(['tk'+o.tkv, 'tcl'+o.tkv])
@@ -380,13 +386,20 @@ def build_tkagg(ext_modules, packages, numerix):
 def build_agg(ext_modules, packages, numerix):
     global BUILT_AGG
     if BUILT_AGG: return # only build it if you you haven't already
-    
 
 
+    agg = ('agg_trans_affine.cpp',
+           'agg_path_storage.cpp',
+           'agg_vcgen_stroke.cpp',
+           'agg_bezier_arc.cpp',
+           'agg_curves.cpp',
+           'agg_vcgen_dash.cpp',
+           'agg_rasterizer_scanline_aa.cpp',
+           )
 
     if numerix in ["numarray","both"]: # Build for numarray
-        deps = ['src/ft2font.cpp', 'src/mplutils.cpp']
-        deps.extend(glob.glob('agg22/src/*.cpp'))
+        deps = ['agg22/src/%s'%name for name in agg]
+        deps.extend(('src/ft2font.cpp', 'src/mplutils.cpp'))
         deps.extend(glob.glob('CXX/*.cxx'))
         deps.extend(glob.glob('CXX/*.c'))
         temp_copy('src/_backend_agg.cpp', 'src/_na_backend_agg.cpp')
@@ -401,8 +414,8 @@ def build_agg(ext_modules, packages, numerix):
         add_ft2font_flags(module)
         ext_modules.append(module)    
     if numerix in ["Numeric","both"]: # Build for Numeric
-        deps = ['src/ft2font.cpp', 'src/mplutils.cpp']
-        deps.extend(glob.glob('agg22/src/*.cpp'))
+        deps = ['agg22/src/%s'%name for name in agg]
+        deps.extend(('src/ft2font.cpp', 'src/mplutils.cpp'))
         deps.extend(glob.glob('CXX/*.cxx'))
         deps.extend(glob.glob('CXX/*.c'))
 
@@ -425,10 +438,17 @@ def build_image(ext_modules, packages, numerix):
     global BUILT_IMAGE
     if BUILT_IMAGE: return # only build it if you you haven't already
 
+    agg = ('agg_trans_affine.cpp',
+           'agg_path_storage.cpp',
+           'agg_rasterizer_scanline_aa.cpp',
+           'agg_image_filters.cpp',
+           'agg_bezier_arc.cpp',
+           )
+
     if numerix in ["numarray","both"]: # Build for numarray
         temp_copy('src/_image.cpp', 'src/_na_image.cpp')
         deps = ['src/_na_image.cpp', 'src/mplutils.cpp'] 
-        deps.extend(glob.glob('agg22/src/*.cpp'))
+        deps.extend(['agg22/src/%s'%name for name in agg])
         deps.extend(glob.glob('CXX/*.cxx'))
         deps.extend(glob.glob('CXX/*.c'))
         module = Extension(
@@ -443,9 +463,10 @@ def build_image(ext_modules, packages, numerix):
     if numerix in ["Numeric","both"]: # Build for Numeric
         temp_copy('src/_image.cpp', 'src/_nc_image.cpp')
         deps = ['src/_nc_image.cpp', 'src/mplutils.cpp'] 
-        deps.extend(glob.glob('agg22/src/*.cpp'))
+        deps.extend(['agg22/src/%s'%name for name in agg])
         deps.extend(glob.glob('CXX/*.cxx'))
         deps.extend(glob.glob('CXX/*.c'))
+
         module = Extension(
             'matplotlib._nc_image',
             deps
@@ -457,6 +478,21 @@ def build_image(ext_modules, packages, numerix):
 
     BUILT_IMAGE = True
 
+def build_swigagg(ext_modules, packages):
+    # setup the swig agg wrapper
+    deps = ['src/agg.cxx',
+            'agg22/src/agg_path_storage.cpp',
+            'agg22/src/agg_bezier_arc.cpp',
+            'agg22/src/agg_trans_affine.cpp',            
+            ]
+    agg = Extension('matplotlib._agg',
+                    deps,
+                    )
+
+    agg.include_dirs.extend(['agg22/include', 'src', 'swig'])
+    agg.libraries.extend(['stdc++', 'm'])
+    ext_modules.append(agg)
+
 def build_transforms(ext_modules, packages, numerix):
     if numerix in ["numarray","both"]: # Build for numarray
         cxx = glob.glob('CXX/*.cxx')
@@ -468,7 +504,9 @@ def build_transforms(ext_modules, packages, numerix):
                              libraries = ['stdc++', 'm'],
                              include_dirs = ['src', '.'],
                              )
+        
         module.extra_compile_args.append("-DNUMARRAY=1")
+        add_base_flags(module)
         ext_modules.append(module)
         
     if numerix in ["Numeric","both"]:  # Build for Numeric
@@ -482,8 +520,24 @@ def build_transforms(ext_modules, packages, numerix):
                              include_dirs = ['src', '.'],
                              )
         module.extra_compile_args.append("-DNUMERIC=1")
+        add_base_flags(module)
         ext_modules.append(module)
-    
+
+
+def build_enthought(ext_modules, packages):
+    global BUILT_ENTHOUGHT
+    if BUILT_ENTHOUGHT: return # only build it if you you haven't already
+
+    ctraits = Extension('matplotlib.enthought.traits.ctraits',  ['lib/matplotlib/enthought/traits/ctraits.c'])
+    ext_modules.append(ctraits)
+    packages.extend(['matplotlib/enthought',
+                     'matplotlib/enthought/traits',
+                     'matplotlib/enthought/traits/ui',
+                     'matplotlib/enthought/traits/ui/null',
+                     'matplotlib/enthought/resource',
+                     'matplotlib/enthought/util',
+                     ])
+    BUILT_ENTHOUGHT = True    
 
 def build_contour(ext_modules, packages, numerix):
     global BUILT_CONTOUR
@@ -499,6 +553,7 @@ def build_contour(ext_modules, packages, numerix):
 		libraries = ['stdc++'],
             )    
         module.extra_compile_args.append('-DNUMARRAY=1')
+        add_base_flags(module)
         ext_modules.append(module)    
 
     if numerix in ["Numeric","both"]: # Build for Numeric
@@ -511,10 +566,40 @@ def build_contour(ext_modules, packages, numerix):
 		libraries = ['stdc++'],
             )
         module.extra_compile_args.append('-DNUMERIC=1')
+        add_base_flags(module)
         ext_modules.append(module)
 
     BUILT_CONTOUR = True
 
 
+def build_gdk(ext_modules, packages, numerix):
+    global BUILT_GDK
+    if BUILT_GDK: return # only build it if you you haven't already
 
+    if numerix in ["numarray","both"]: # Build for numarray
+        temp_copy('src/_backend_gdk.c', 'src/_na_backend_gdk.c')
+        module = Extension(
+            'matplotlib._na_backend_gdk',
+		[ 'src/_na_backend_gdk.c',
+		],
+		libraries = [],
+            )
+        module.extra_compile_args.append('-DNUMARRAY=1')
+        add_base_flags(module)
+        add_pygtk_flags(module)        
+        ext_modules.append(module)
 
+    if numerix in ["Numeric","both"]: # Build for Numeric
+        temp_copy('src/_backend_gdk.c', 'src/_nc_backend_gdk.c')
+        module = Extension(
+            'matplotlib._nc_backend_gdk',
+		[ 'src/_nc_backend_gdk.c',
+		],
+		libraries = [],
+            )
+        module.extra_compile_args.append('-DNUMERIC=1')
+        add_base_flags(module)
+        add_pygtk_flags(module)        
+        ext_modules.append(module)
+
+    BUILT_GDK = True
