@@ -79,7 +79,7 @@ protected:
   void _set_antialiased( const Py::Object& gc);
 };
 
-// the renderer
+// th renderer
 class RendererAgg: public Py::PythonExtension<RendererAgg> {
   typedef std::pair<bool, agg::rgba> facepair_t;
 public:
@@ -96,6 +96,9 @@ public:
   Py::Object draw_poly_collection(const Py::Tuple& args);
   Py::Object draw_regpoly_collection(const Py::Tuple& args);
   Py::Object draw_lines(const Py::Tuple & args);
+  Py::Object draw_path(const Py::Tuple & args);
+  Py::Object _draw_markers_nocache(const Py::Tuple & args);
+  Py::Object _draw_markers_cache(const Py::Tuple & args);
   Py::Object draw_markers(const Py::Tuple & args);
   Py::Object draw_text(const Py::Tuple & args);
   Py::Object draw_image(const Py::Tuple & args);
@@ -138,7 +141,7 @@ protected:
   agg::rgba rgb_to_color(const Py::SeqBase<Py::Object>& rgb, double alpha);
   facepair_t _get_rgba_face(const Py::Object& rgbFace, double alpha);
   void set_clipbox_rasterizer( double *cliprect);
-  template <class VS> void _fill_and_stroke(VS&, const GCAgg&, const facepair_t&);  
+  template <class VS> void _fill_and_stroke(VS&, const GCAgg&, const facepair_t&, bool curvy=true);  
 
   void _render_lines_path(agg::path_storage &ps, const GCAgg& gc);
 };
