@@ -44,7 +44,7 @@ LazyValue::compare(const Py::Object &other) {
   else ret=1;
   return ret;
 }
-
+ 
 Py::Object 
 LazyValue::number_add( const Py::Object &o ) {
   _VERBOSE("LazyValue::number");
@@ -132,7 +132,7 @@ Point::~Point()
 }
 
 Interval::Interval(LazyValue* val1, LazyValue* val2) : 
-  _val1(val1), _val2(val2) {
+  _val1(val1), _val2(val2), _minpos(NULL) {
   _VERBOSE("Interval::Interval");
   Py_INCREF(val1);
   Py_INCREF(val2);
@@ -170,10 +170,10 @@ Interval::update(const Py::Tuple &args) {
 
   for (size_t i=0; i<Nval; ++i) {
     thisval = Py::Float(vals[i]);
-
     if (thisval<minx) minx = thisval;
     if (thisval>maxx) maxx = thisval;
-  } 
+    _minpos->update(thisval);
+  }  
 
 
   _val1->set_api(minx);
@@ -183,6 +183,7 @@ Interval::update(const Py::Tuple &args) {
 
 Bbox::Bbox(Point* ll, Point* ur) : _ll(ll), _ur(ur) {
   _VERBOSE("Bbox::Bbox");
+  
   Py_INCREF(ll);
   Py_INCREF(ur);
 };
@@ -264,6 +265,34 @@ Bbox::get_bounds(const Py::Tuple & args) {
   return ret;
 }
 
+Py::Object
+Bbox::count_contains(const Py::Tuple &args) {
+  _VERBOSE("Bbox::count_contains");
+  args.verify_length(1);
+
+  Py::SeqBase<Py::Object> xys = args[0];
+  size_t Nxys = xys.length();
+  long count = 0;
+
+  double minx = _ll->xval();
+  double miny = _ll->yval();  
+  double maxx = _ur->xval();
+  double maxy = _ur->yval();
+
+  for(size_t i=0; i < Nxys; i++) {
+    Py::SeqBase<Py::Object> xy(xys[i]);
+    xy.verify_length(2);
+    double x = Py::Float(xy[0]);
+    double y = Py::Float(xy[1]);
+    int inx = ( (x>=minx) && (x<=maxx) || (x>=maxx) && (x<=minx) );
+    if (!inx) continue;
+    int iny = ( (y>=miny) && (y<=maxy) || (y>=maxy) && (y<=miny) );
+    if (!iny) continue;
+    count += 1;
+  }
+  return Py::Int(count);
+}
+
 Py::Object 
 Bbox::contains(const Py::Tuple &args) {
   _VERBOSE("Bbox::contains");
@@ -377,12 +406,13 @@ Bbox::update(const Py::Tuple &args) {
     tup = xys[i];
     double x = Py::Float(tup[0]);
     double y = Py::Float(tup[1]);
-
+    _posx.update(x);
+    _posy.update(y);
     if (x<minx) minx=x;
     if (x>maxx) maxx=x;
     if (y<miny) miny=y;
     if (y>maxy) maxy=y;
-
+    
   } 
 
 
@@ -393,6 +423,80 @@ Bbox::update(const Py::Tuple &args) {
   return Py::Object();
 }
 
+Py::Object 
+Bbox::update_numerix(const Py::Tuple &args) {
+  //update the boox from the numerix arrays x and y
+  _VERBOSE("Bbox::update_numerix");
+
+  args.verify_length(3);
+
+  Py::Object xo = args[0];
+  Py::Object yo = args[1];
+
+  PyArrayObject *x = (PyArrayObject *) PyArray_ContiguousFromObject(xo.ptr(), PyArray_DOUBLE, 1, 1); 
+  
+  if (x==NULL) 
+    throw Py::TypeError("Bbox::update_numerix expected numerix array");
+
+  
+  PyArrayObject *y = (PyArrayObject *) PyArray_ContiguousFromObject(yo.ptr(), PyArray_DOUBLE, 1, 1); 
+
+  if (y==NULL) 
+    throw Py::TypeError("Bbox::update_numerix expected numerix array");
+
+  
+  size_t Nx = x->dimensions[0];
+  size_t Ny = y->dimensions[0];
+  
+  if (Nx!=Ny) 
+    throw Py::ValueError("x and y must be equal length sequences");
+
+  //don't use current bounds when updating box if ignore==1
+
+
+  if (Nx==0) return Py::Object();
+
+  double minx = _ll->xval();
+  double maxx = _ur->xval();
+  double miny = _ll->yval();
+  double maxy = _ur->yval();
+
+  double thisx, thisy;
+  int ignore = Py::Int(args[2]);  
+  if (ignore) {
+    thisx = *(double *)(x->data);
+    thisy = *(double *)(y->data);
+    minx=thisx;
+    maxx=thisx;
+    miny=thisy;
+    maxy=thisy;
+  }
+
+  for (size_t i=0; i< Nx; ++i) {
+
+    thisx = *(double *)(x->data + i*x->strides[0]);
+    thisy = *(double *)(y->data + i*y->strides[0]);
+    _posx.update(thisx);
+    _posy.update(thisy);
+    if (thisx<minx) minx=thisx;
+    if (thisx>maxx) maxx=thisx;
+    if (thisy<miny) miny=thisy;
+    if (thisy>maxy) maxy=thisy;
+
+
+  }
+
+  Py_XDECREF(x);
+  Py_XDECREF(y);
+ 
+
+  _ll->x_api()->set_api(minx);
+  _ll->y_api()->set_api(miny);
+  _ur->x_api()->set_api(maxx);
+  _ur->y_api()->set_api(maxy);
+  return Py::Object();
+}
+
 Func::~Func() {
   _VERBOSE("Func::~Func");
 }
@@ -405,7 +509,14 @@ Func::map(const Py::Tuple &args) {
   args.verify_length(1);
   double xin = Py::Float(args[0]);
   
-  double xout = this->operator()(xin);
+  double xout;
+  try {
+    xout = this->operator()(xin);
+  }
+  catch(...) {
+    throw Py::ValueError("Domain error on Func::map");  
+  }
+
   return Py::Float(xout);
 
 };
@@ -421,19 +532,6 @@ Func::inverse(const Py::Tuple &args) {
   return Py::Float(xout);
 };
 
-FuncXY::FuncXY(Func* funcx, Func* funcy) : 
-  _funcx(funcx), _funcy(funcy) {
-  _VERBOSE("FuncXY::FuncXY");
-    Py_INCREF(funcx);
-    Py_INCREF(funcy);
-}
-
-FuncXY::~FuncXY() {
-  _VERBOSE("FuncXY::~FuncXY");
-  Py_DECREF(_funcx);
-  Py_DECREF(_funcy);
-
-}
 
 Py::Object 
 FuncXY::map(const Py::Tuple &args) {
@@ -443,7 +541,13 @@ FuncXY::map(const Py::Tuple &args) {
   double xin = Py::Float(args[0]);
   double yin = Py::Float(args[1]);
   
-  std::pair<double, double> xy = this->operator()(xin, yin);
+  std::pair<double, double> xy;
+  try {
+    xy = this->operator()(xin, yin);
+  }
+  catch(...) {
+    throw Py::ValueError("Domain error on FuncXY nonlinear transform");  
+  }
 
   Py::Tuple ret(2);
   double xout = xy.first;
@@ -475,51 +579,6 @@ FuncXY::inverse(const Py::Tuple &args) {
 };
 
 
-
-Py::Object
-FuncXY::set_funcx(const Py::Tuple & args) {
-  _VERBOSE("FuncXY::set_funcx");
-  args.verify_length(1);
-  
-  if (!Func::check(args[0])) 
-    throw Py::TypeError("set_funcx(func) expected a Func instance");
-  
-  _funcx = static_cast<Func*>(args[0].ptr());
-  Py_INCREF(_funcx);
-  return Py::Object();
-}
-
-Py::Object
-FuncXY::set_funcy(const Py::Tuple & args) {
-  _VERBOSE("FuncXY::set_funcy");
-  args.verify_length(1);
-  
-  if (!Func::check(args[0])) 
-    throw Py::TypeError("set_funcy(func) expected a Func instance");
-  
-  _funcy = static_cast<Func*>(args[0].ptr());
-  Py_INCREF(_funcy);
-  return Py::Object();
-}
-
-Py::Object
-FuncXY::get_funcx(const Py::Tuple & args) {
-  _VERBOSE("FuncXY::get_funcx");
-  args.verify_length(0);
-  return Py::Object(_funcx);
-}
-
-Py::Object
-FuncXY::get_funcy(const Py::Tuple & args) {
-  _VERBOSE("FuncXY::get_funcy");
-  args.verify_length(0);
-  return Py::Object(_funcy);
-}
-
-PolarXY::~PolarXY() {
-  _VERBOSE("PolarXY::~PolarXY");
-}
-
 Transformation::~Transformation() {
   _VERBOSE("Transformation::~Transformation");
   if (_transOffset!=NULL) {
@@ -536,6 +595,7 @@ Transformation::as_vec6(const Py::Tuple & args) {
 }
 
 
+
 Py::Object
 Transformation::get_funcx(const Py::Tuple & args) {
   _VERBOSE("Transformation::get_funcx");
@@ -566,6 +626,20 @@ Transformation::set_funcy(const Py::Tuple & args) {
 }
 
 
+Py::Object
+Transformation::get_funcxy(const Py::Tuple & args) {
+  _VERBOSE("Transformation::get_funcxy");
+  throw Py::RuntimeError("This transformation does not support get_funcxy");
+  return Py::Object();
+}
+
+
+Py::Object
+Transformation::set_funcxy(const Py::Tuple & args) {
+  _VERBOSE("Transformation::set_funcxy");
+  throw Py::RuntimeError("This transformation does not support set_funcxy");
+  return Py::Object();
+}
 
 Py::Object
 Transformation::get_bbox1(const Py::Tuple & args) {
@@ -630,7 +704,13 @@ Transformation::inverse_xy_tup(const Py::Tuple & args) {
   double xin = Py::Float(tup[0]);
   double yin = Py::Float(tup[1]);
 
-  if (!_frozen) eval_scalars();
+  try {
+    if (!_frozen) eval_scalars();
+  }
+  catch(...) {
+    throw Py::ValueError("Domain error on invser_xy_tup");  
+  }
+
   
   inverse_api(xin, yin);
   Py::Tuple ret(2);
@@ -645,7 +725,13 @@ Transformation::xy_tup(const Py::Tuple & args) {
   _VERBOSE("Transformation::xy_tup");
   args.verify_length(1);
 
-  if (!_frozen) eval_scalars();
+  try {
+    if (!_frozen) eval_scalars();
+  }
+  catch(...) {
+    throw Py::ValueError("Domain error on nonlinear transform");  
+  }
+
 
   Py::SeqBase<Py::Object> xytup = args[0];
   double x = Py::Float(xytup[0]);
@@ -653,7 +739,13 @@ Transformation::xy_tup(const Py::Tuple & args) {
   
 
   Py::Tuple out(2);
-  this->operator()(x, y);
+  try {
+    this->operator()(x, y);
+  }
+  catch(...) {
+    throw Py::ValueError("Domain error on nTransformation::xy_tup operator()(x,y)");  
+  }
+
   out[0] = Py::Float( xy.first ); 
   out[1] = Py::Float( xy.second ); 
   return out;
@@ -674,7 +766,13 @@ Transformation::seq_x_y(const Py::Tuple & args) {
     throw Py::ValueError("x and y must be equal length sequences");
 
   // evaluate the lazy objects  
-  if (!_frozen) eval_scalars();
+  try {
+    if (!_frozen) eval_scalars();
+  }
+  catch(...) {
+    throw Py::ValueError("Domain error on Transformation::seq_x_y");  
+  }
+
 
   Py::Tuple xo(Nx);
   Py::Tuple yo(Nx);
@@ -683,7 +781,13 @@ Transformation::seq_x_y(const Py::Tuple & args) {
   for (size_t i=0; i< Nx; ++i) {
     double thisx = Py::Float(x[i]);
     double thisy = Py::Float(y[i]);
-    this->operator()(thisx, thisy);
+    try {
+      this->operator()(thisx, thisy);
+    }
+    catch(...) {
+      throw Py::ValueError("Domain error on Transformation::seq_x_y operator()(thisx, thisy)");  
+    }
+
     xo[i] = Py::Float( xy.first );
     yo[i] = Py::Float( xy.second );
   }
@@ -722,7 +826,13 @@ Transformation::numerix_x_y(const Py::Tuple & args) {
     throw Py::ValueError("x and y must be equal length sequences");
 
   // evaluate the lazy objects  
-  if (!_frozen) eval_scalars();
+  try {
+    if (!_frozen) eval_scalars();
+  }
+  catch(...) {
+    throw Py::ValueError("Domain error on Transformation::numerix_x_y");  
+  }
+
 
   int dimensions[1];
   dimensions[0] = Nx;
@@ -747,7 +857,13 @@ Transformation::numerix_x_y(const Py::Tuple & args) {
     double thisx = *(double *)(x->data + i*x->strides[0]);
     double thisy = *(double *)(y->data + i*y->strides[0]);
     //std::cout << "calling operator " << thisx << " " << thisy << " " << std::endl;
-    this->operator()(thisx, thisy);
+    try {
+      this->operator()(thisx, thisy);
+    }
+    catch(...) {
+      throw Py::ValueError("Domain error on Transformation::numerix_x_y");  
+    }
+
     *(double *)(retx->data + i*retx->strides[0]) = xy.first;
     *(double *)(rety->data + i*rety->strides[0]) = xy.second;
   }
@@ -763,6 +879,78 @@ Transformation::numerix_x_y(const Py::Tuple & args) {
   return ret;
 }
 
+Py::Object
+Transformation::nonlinear_only_numerix(const Py::Tuple & args) {
+  _VERBOSE("Transformation::nonlinear_only_numerix");
+  args.verify_length(2);
+
+
+  Py::Object xo = args[0];
+  Py::Object yo = args[1];
+
+  PyArrayObject *x = (PyArrayObject *) PyArray_ContiguousFromObject(xo.ptr(), PyArray_DOUBLE, 1, 1); 
+  
+  if (x==NULL) 
+    throw Py::TypeError("Transformation::nonlinear_only_numerix expected numerix array");
+
+  
+  PyArrayObject *y = (PyArrayObject *) PyArray_ContiguousFromObject(yo.ptr(), PyArray_DOUBLE, 1, 1); 
+  
+  if (y==NULL) 
+    throw Py::TypeError("Transformation::nonlinear_only_numerix expected numerix array");
+
+  
+  size_t Nx = x->dimensions[0];
+  size_t Ny = y->dimensions[0];
+  
+  if (Nx!=Ny) 
+    throw Py::ValueError("x and y must be equal length sequences");
+
+  int dimensions[1];
+  dimensions[0] = Nx;
+
+  
+  PyArrayObject *retx = (PyArrayObject *)PyArray_FromDims(1,dimensions,PyArray_DOUBLE);
+  if (retx==NULL) {
+    Py_XDECREF(x);
+    Py_XDECREF(y);
+    throw Py::RuntimeError("Could not create return x array");
+  }
+
+  PyArrayObject *rety = (PyArrayObject *)PyArray_FromDims(1,dimensions,PyArray_DOUBLE);
+  if (rety==NULL) {
+    Py_XDECREF(x);
+    Py_XDECREF(y);
+    throw Py::RuntimeError("Could not create return x array");
+  }
+
+  for (size_t i=0; i< Nx; ++i) {
+
+    double thisx = *(double *)(x->data + i*x->strides[0]);
+    double thisy = *(double *)(y->data + i*y->strides[0]);
+    //std::cout << "calling operator " << thisx << " " << thisy << " " << std::endl;
+    try {
+      this->nonlinear_only_api(&thisx, &thisy);
+    }
+    catch(...) {
+      throw Py::ValueError("Domain error on this->nonlinear_only_api(&thisx, &thisy) in Transformation::nonlinear_only_numerix");  
+    }
+
+    *(double *)(retx->data + i*retx->strides[0]) = thisx;
+    *(double *)(rety->data + i*rety->strides[0]) = thisy;
+  }
+  
+  Py_XDECREF(x);
+  Py_XDECREF(y);
+
+  Py::Tuple ret(2);
+  ret[0] = Py::Object((PyObject*)retx);
+  ret[1] = Py::Object((PyObject*)rety);
+  Py_XDECREF(retx);
+  Py_XDECREF(rety);
+  return ret;
+}
+
 Py::Object
 Transformation::seq_xy_tups(const Py::Tuple & args) {
   _VERBOSE("Transformation::seq_xy_tups");
@@ -770,10 +958,16 @@ Transformation::seq_xy_tups(const Py::Tuple & args) {
   
   Py::SeqBase<Py::Object> xytups = args[0];
   
+
   size_t Nx = xytups.length();
 
-  if (!_frozen) eval_scalars();
-
+  try {
+    if (!_frozen) eval_scalars();
+  }
+  catch(...) {
+    throw Py::ValueError("Domain error on Transformation::seq_xy_tups");  
+  }
+  
   Py::Tuple ret(Nx);
   Py::SeqBase<Py::Object> xytup;
   
@@ -785,7 +979,13 @@ Transformation::seq_xy_tups(const Py::Tuple & args) {
     double thisx = Py::Float(xytup[0]);
     double thisy = Py::Float(xytup[1]);
 
-    this->operator()(thisx, thisy);
+    try {
+      this->operator()(thisx, thisy);
+    }
+    catch(...) {
+      throw Py::ValueError("Domain error on nonlinear Transformation::seq_xy_tups operator()(thisx, thisy)");  
+    }
+
 
     Py::Tuple out(2);
     out[0] = Py::Float( xy.first );
@@ -797,12 +997,82 @@ Transformation::seq_xy_tups(const Py::Tuple & args) {
 }
 
 
-SeparableTransformation::SeparableTransformation(Bbox *b1, Bbox *b2, Func *funcx, Func *funcy) : 
+BBoxTransformation::BBoxTransformation(Bbox *b1, Bbox *b2) : 
     Transformation(), 
-    _b1(b1), _b2(b2), _funcx(funcx), _funcy(funcy)  {
-  _VERBOSE("SeparableTransformation::SeparableTransformation");
+    _b1(b1), _b2(b2)  {
+  _VERBOSE("BBoxTransformation::BBoxTransformation");
   Py_INCREF(b1);
   Py_INCREF(b2);
+  
+}
+
+BBoxTransformation::~BBoxTransformation() {
+  _VERBOSE("BBoxTransformation::~BBoxTransformation");
+  Py_DECREF(_b1);
+  Py_DECREF(_b2);
+}
+
+Py::Object
+BBoxTransformation::get_bbox1(const Py::Tuple & args) {
+  _VERBOSE("BBoxTransformation::get_bbox1");
+  args.verify_length(0);
+  return Py::Object(_b1);
+}
+
+Py::Object
+BBoxTransformation::get_bbox2(const Py::Tuple & args) {
+  _VERBOSE("BBoxTransformation::get_bbox2");
+  args.verify_length(0);
+  return Py::Object(_b2);
+}
+
+
+Py::Object
+BBoxTransformation::set_bbox1(const Py::Tuple & args) {
+  _VERBOSE("BBoxTransformation::set_bbox1");
+  args.verify_length(1);
+  if (!Bbox::check(args[0])) 
+    throw Py::TypeError("set_bbox1(func) expected a func instance");
+  _b1 = static_cast<Bbox*>(args[0].ptr());
+  Py_INCREF(_b1);
+  return Py::Object();
+}
+
+Py::Object
+BBoxTransformation::set_bbox2(const Py::Tuple & args) {
+  _VERBOSE("BBoxTransformation::set_bbox2");
+  args.verify_length(1);
+  if (!Bbox::check(args[0])) 
+    throw Py::TypeError("set_bbox2(func) expected a func instance");
+  _b2 = static_cast<Bbox*>(args[0].ptr());
+  Py_INCREF(_b2);
+  return Py::Object();
+}
+
+void
+BBoxTransformation::affine_params_api(double* a, double* b, double* c, double* d, double* tx, double* ty) {
+  //get the scale and translation factors of the separable transform
+  //sx, sy, tx, ty
+    if (!_frozen) eval_scalars();
+    *a = _sx;
+    *b = 0.0;
+    *c = 0.0;
+    *d = _sy;
+    
+    *tx = _tx;
+    *ty = _ty;
+
+    if (_usingOffset) {
+      *tx  += _xot;
+      *ty  += _yot;
+    }
+
+}
+
+SeparableTransformation::SeparableTransformation(Bbox *b1, Bbox *b2, Func *funcx, Func *funcy) : 
+    BBoxTransformation(b1, b2), 
+    _funcx(funcx), _funcy(funcy)  {
+  _VERBOSE("SeparableTransformation::SeparableTransformation");
   Py_INCREF(funcx);
   Py_INCREF(funcy);
   
@@ -811,8 +1081,6 @@ SeparableTransformation::SeparableTransformation(Bbox *b1, Bbox *b2, Func *funcx
 
 SeparableTransformation::~SeparableTransformation() {
   _VERBOSE("SeparableTransformation::~SeparableTransformation");
-  Py_DECREF(_b1);
-  Py_DECREF(_b2);
   Py_DECREF(_funcx);
   Py_DECREF(_funcy);
 }
@@ -856,49 +1124,22 @@ SeparableTransformation::set_funcy(const Py::Tuple & args) {
 
 
 
-Py::Object
-SeparableTransformation::get_bbox1(const Py::Tuple & args) {
-  _VERBOSE("SeparableTransformation::get_bbox1");
-  args.verify_length(0);
-  return Py::Object(_b1);
+
+void 
+SeparableTransformation::nonlinear_only_api(double *x, double *y) {
+
+  double thisx = _funcx->operator()(*x);
+  double thisy = _funcy->operator()(*y);
+  *x = thisx;
+  *y = thisy;
 }
 
-Py::Object
-SeparableTransformation::get_bbox2(const Py::Tuple & args) {
-  _VERBOSE("SeparableTransformation::get_bbox2");
-  args.verify_length(0);
-  return Py::Object(_b2);
-}
-
-
-Py::Object
-SeparableTransformation::set_bbox1(const Py::Tuple & args) {
-  _VERBOSE("SeparableTransformation::set_bbox1");
-  args.verify_length(1);
-  if (!Bbox::check(args[0])) 
-    throw Py::TypeError("set_bbox1(func) expected a func instance");
-  _b1 = static_cast<Bbox*>(args[0].ptr());
-  Py_INCREF(_b1);
-  return Py::Object();
-}
-
-Py::Object
-SeparableTransformation::set_bbox2(const Py::Tuple & args) {
-  _VERBOSE("SeparableTransformation::set_bbox2");
-  args.verify_length(1);
-  if (!Bbox::check(args[0])) 
-    throw Py::TypeError("set_bbox2(func) expected a func instance");
-  _b2 = static_cast<Bbox*>(args[0].ptr());
-  Py_INCREF(_b2);
-  return Py::Object();
-}
-
-
 std::pair<double, double>&
 SeparableTransformation::operator()(const double& x, const double& y) {
   _VERBOSE("SeparableTransformation::operator");
 
   // calling function must first call eval_scalars
+  
   double fx = _funcx->operator()(x);
   double fy = _funcy->operator()(y);
   
@@ -942,13 +1183,19 @@ SeparableTransformation::inverse_api(const double &x, const double &y) {
 }
 
 
+
 void 
 SeparableTransformation::eval_scalars(void) {
   _VERBOSE("SeparableTransformation::eval_scalars");
-  double xminIn  = _funcx->operator()( _b1->ll_api()->xval() );
-  double xmaxIn  = _funcx->operator()( _b1->ur_api()->xval() );
-  double yminIn  = _funcy->operator()( _b1->ll_api()->yval() );
-  double ymaxIn  = _funcy->operator()( _b1->ur_api()->yval() );
+  double xminIn;
+  double xmaxIn;
+  double yminIn;
+  double ymaxIn;
+  
+  xminIn  = _funcx->operator()( _b1->ll_api()->xval() );
+  xmaxIn  = _funcx->operator()( _b1->ur_api()->xval() );
+  yminIn  = _funcy->operator()( _b1->ll_api()->yval() );
+  ymaxIn  = _funcy->operator()( _b1->ur_api()->yval() );
 
   double xminOut  = _b2->ll_api()->xval();
   double xmaxOut  = _b2->ur_api()->xval();
@@ -960,7 +1207,7 @@ SeparableTransformation::eval_scalars(void) {
 
   double heightIn  = ymaxIn  - yminIn;
   double heightOut = ymaxOut - yminOut;
-
+  //std::cout <<"heightout, heightin = "  << heightOut << " " << heightIn << std::endl;
   if (widthIn==0) 
     throw Py::ZeroDivisionError("SeparableTransformation::eval_scalars xin interval is zero; cannot transform");
 
@@ -989,10 +1236,11 @@ SeparableTransformation::eval_scalars(void) {
   }
 
   if (_usingOffset) {
-    _transOffset->eval_scalars();
-    _transOffset->operator()(_xo, _yo);
-    _xot = _transOffset->xy.first;
-    _yot = _transOffset->xy.second;
+    
+      _transOffset->eval_scalars();
+      _transOffset->operator()(_xo, _yo);
+      _xot = _transOffset->xy.first;
+      _yot = _transOffset->xy.second;
   }
 }
 
@@ -1004,6 +1252,165 @@ SeparableTransformation::deepcopy(const Py::Tuple &args) {
   return Py::asObject( new SeparableTransformation( static_cast<Bbox*>((_b1->_deepcopy()).ptr()),static_cast<Bbox*>((_b2->_deepcopy()).ptr()), _funcx,_funcy ));
 }
 
+NonseparableTransformation::NonseparableTransformation(Bbox *b1, Bbox *b2, FuncXY *funcxy) : 
+    BBoxTransformation(b1, b2), 
+    _funcxy(funcxy)  {
+  _VERBOSE("NonseparableTransformation::NonseparableTransformation");
+  Py_INCREF(funcxy);
+}
+
+
+NonseparableTransformation::~NonseparableTransformation() {
+  _VERBOSE("NonseparableTransformation::~NonseparableTransformation");
+  Py_DECREF(_funcxy);
+}
+
+Py::Object
+NonseparableTransformation::get_funcxy(const Py::Tuple & args) {
+  _VERBOSE("NonseparableTransformation::get_funcxy");
+  args.verify_length(0);
+  return Py::Object(_funcxy);
+}
+ 
+
+Py::Object
+NonseparableTransformation::set_funcxy(const Py::Tuple & args) {
+  _VERBOSE("NonseparableTransformation::set_funcx");
+  args.verify_length(1);
+  if (!FuncXY::check(args[0])) 
+    throw Py::TypeError("set_funcxy(func) expected a func instance");
+  _funcxy = static_cast<FuncXY*>(args[0].ptr());
+  Py_INCREF(_funcxy);
+  return Py::Object();
+}
+
+void 
+NonseparableTransformation::nonlinear_only_api(double *x, double *y) {
+
+  xy = _funcxy->operator()(*x,*y);
+  *x = xy.first;
+  *y = xy.second;
+}
+
+
+std::pair<double, double>&
+NonseparableTransformation::operator()(const double& x, const double& y) {
+  _VERBOSE("NonseparableTransformation::operator");
+ 
+  // calling function must first call eval_scalars
+  xy = _funcxy->operator()(x,y);
+  
+  //std::cout << "operator(x,y) In: " << x << " " << y << " " << xy.first << " " << xy.second << std::endl;
+  xy.first  = _sx * xy.first  +  _tx ;
+  xy.second = _sy * xy.second +  _ty;
+  //std::cout << "operator(x,y) out: " << xy.first << " " << xy.second << std::endl;
+  if (_usingOffset) {
+    xy.first  += _xot;
+    xy.second += _yot;
+  }
+  
+
+  return xy;
+}
+
+
+std::pair<double, double> &
+NonseparableTransformation::inverse_api(const double &x, const double &y) {
+  _VERBOSE("NonseparableTransformation::inverse_api");
+
+  // calling function must first call eval_scalars_inverse and
+  // _transOffset->eval_scalars_inverse()
+
+
+  if (!_invertible)
+    throw Py::RuntimeError("Transformation is not invertible");
+
+  double xin = x;
+  double yin = y;
+
+  if (_usingOffset) {
+    xin  -= _xot;
+    yin  -= _yot;
+  }
+
+  xy  = _funcxy->inverse_api( _isx * xin  +  _itx, 
+			      _isy * yin  +  _ity );
+     
+  return xy;
+}
+
+
+void 
+NonseparableTransformation::eval_scalars(void) {
+  _VERBOSE("NonseparableTransformation::eval_scalars");
+  std::pair<double, double> xyminIn = _funcxy->
+    operator()( _b1->ll_api()->xval(), _b1->ll_api()->yval());
+
+  std::pair<double, double> xymaxIn = _funcxy->
+    operator()( _b1->ur_api()->xval(), _b1->ur_api()->yval());
+
+  std::pair<double, double> xyminOut( _b2->ll_api()->xval(), _b2->ll_api()->yval() );
+
+  std::pair<double, double> xymaxOut( _b2->ur_api()->xval(), _b2->ur_api()->yval() );
+
+	       
+
+  double widthIn  = xymaxIn.first  - xyminIn.first;
+  double widthOut = xymaxOut.first - xyminOut.first;
+
+  double heightIn  = xymaxIn.second  - xyminIn.second;
+  double heightOut = xymaxOut.second - xyminOut.second;
+
+  if (widthIn==0) 
+    throw Py::ZeroDivisionError("NonseparableTransformation::eval_scalars xin interval is zero; cannot transform");
+
+  if (heightIn==0)
+    throw Py::ZeroDivisionError("NonseparableTransformation::eval_scalars yin interval is zero; cannot transform");
+ 
+
+
+  _sx = widthOut/widthIn;
+  _sy = heightOut/heightIn;
+
+  _tx = -xyminIn.first*_sx + xyminOut.first;
+  _ty = -xyminIn.second*_sy + xyminOut.second;
+
+  /*
+ std::cout <<"corners in "  
+	   << xyminIn.first << " " << xyminIn.second <<  " " 
+	   << xymaxIn.first << " " << xymaxIn.second <<  std::endl;
+ std::cout <<"w,h in "  << widthIn << " " << heightIn <<  std::endl;
+ std::cout <<"heightout, heightin = "  << heightOut << " " << heightIn << std::endl;
+ std::cout <<"sx,sy,tx,ty = "  << _sx << " " << _sy <<  " " << _tx << " " << _ty << std::endl;
+  */ 
+  //now do the inverse mapping
+  if ( (widthOut==0) || (widthOut==0) ) {
+    _invertible = false;
+  }
+  else {
+    _isx = widthIn/widthOut;
+    _isy = heightIn/heightOut;
+  
+    _itx = -xyminOut.first*_isx + xyminIn.first;
+    _ity = -xyminOut.second*_isy + xyminIn.second;
+  }
+
+  if (_usingOffset) {
+    _transOffset->eval_scalars();
+    _transOffset->operator()(_xo, _yo);
+    _xot = _transOffset->xy.first;
+    _yot = _transOffset->xy.second;
+  }
+}
+
+
+Py::Object 
+NonseparableTransformation::deepcopy(const Py::Tuple &args) {
+  _VERBOSE("NonseparableTransformation::deepcopy");
+  args.verify_length(0);
+  return Py::asObject( new NonseparableTransformation( static_cast<Bbox*>((_b1->_deepcopy()).ptr()),static_cast<Bbox*>((_b2->_deepcopy()).ptr()), _funcxy ));
+}
+
 Affine::Affine(LazyValue *a, LazyValue *b,  LazyValue *c, 
 	       LazyValue *d, LazyValue *tx, LazyValue *ty) : 
   _a(a), _b(b), _c(c), _d(d), _tx(tx), _ty(ty) {
@@ -1028,6 +1435,17 @@ Affine::~Affine() {
 }
 
 
+void
+Affine::affine_params_api(double* a, double* b, double* c, double*d, double* tx, double* ty) {
+
+  *a = _a->val();
+  *b = _b->val();
+  *c = _c->val();
+  *d = _d->val();
+  *tx = _tx->val();
+  *ty = _ty->val();
+
+}  
 Py::Object 
 Affine::as_vec6(const Py::Tuple &args) {
   _VERBOSE("Affine::as_vec6");
@@ -1114,13 +1532,21 @@ Affine::eval_scalars(void) {
     _xot = _transOffset->xy.first;
     _yot = _transOffset->xy.second;
   }
+
+  _VERBOSE("Affine::eval_scalars DONE");
 }
 
 Py::Object 
 Affine::deepcopy(const Py::Tuple &args) {
   _VERBOSE("Affine::deepcopy");
   args.verify_length(0);
-  eval_scalars();
+  try {
+    eval_scalars();
+  }
+  catch(...) {
+    throw Py::ValueError("Domain error on Affine deepcopy");  
+  }
+
   return Py::asObject( new Affine( new Value(_aval),new Value(_bval), new Value(_cval),
                                    new Value(_dval),new Value(_txval),new Value(_tyval) ));
 }
@@ -1238,25 +1664,12 @@ Py::Object
 _transforms_module::new_funcxy (const Py::Tuple &args)
 {
   _VERBOSE("_transforms_module::new_funcxy ");
-  args.verify_length(2);
-  if (!Func::check(args[0]))
-    throw Py::TypeError("FuncXY(funcx, funcy) expected a Func instance for funcx)");
-  if (!Func::check(args[1]))
-    throw Py::TypeError("FuncXY(funcx, funcy) expected a Func instance for funcy)");
-  Func* funcx  = static_cast<Func*>(args[0].ptr());
-  Func* funcy  = static_cast<Func*>(args[1].ptr());
-
-  return Py::asObject(new FuncXY(funcx, funcy));
+  args.verify_length(1);
+  int typecode = Py::Int(args[0]);
+  return Py::asObject(new FuncXY(typecode));
 }   
 
 
-Py::Object 
-_transforms_module::new_polarxy (const Py::Tuple &args)
-{
-  _VERBOSE("_transforms_module::new_polarxy ");
-  args.verify_length(0);
-  return Py::asObject( new PolarXY() );
-}   
 
 Py::Object 
 _transforms_module::new_separable_transformation (const Py::Tuple &args)
@@ -1281,6 +1694,26 @@ _transforms_module::new_separable_transformation (const Py::Tuple &args)
   return Py::asObject( new SeparableTransformation(box1, box2, funcx, funcy) );
 }     
 
+Py::Object 
+_transforms_module::new_nonseparable_transformation (const Py::Tuple &args)
+{
+  _VERBOSE("_transforms_module::new_nonseparable_transformation ");
+  args.verify_length(3);
+  if (!Bbox::check(args[0]))
+    throw Py::TypeError("NonseparableTransform(box1, box2, funcxy) expected a Bbox for box1");
+  if (!Bbox::check(args[1]))
+    throw Py::TypeError("NonseparableTransform(box1, box2, funcxy) expected a Bbox for box2");
+  if (!FuncXY::check(args[2]))
+    throw Py::TypeError("NonseparableTransform(box1, box2, funcxy, funcy) expected a FuncXY for funcxy");
+
+
+  Bbox* box1  = static_cast<Bbox*>(args[0].ptr());
+  Bbox* box2  = static_cast<Bbox*>(args[1].ptr());
+  FuncXY* funcxy  = static_cast<FuncXY*>(args[2].ptr()); 
+  
+  return Py::asObject( new NonseparableTransformation(box1, box2, funcxy) );
+}     
+
 void 
 LazyValue::init_type()
 {
@@ -1339,6 +1772,8 @@ Interval::init_type()
   add_varargs_method("span", &Interval::span, "span()\n");
   add_varargs_method("val1", &Interval::val1, "val1()\n");
   add_varargs_method("val2", &Interval::val2, "val2()\n");
+  add_varargs_method("minpos", &Interval::minpos, "minpos()\n");
+
 }
 
 void 
@@ -1351,6 +1786,7 @@ Bbox::init_type()
   add_varargs_method("ll", 	&Bbox::ll, "ll()\n");
   add_varargs_method("ur", 	&Bbox::ur, "ur()\n");
   add_varargs_method("contains" , &Bbox::contains, "contains(x,y)\n");
+  add_varargs_method("count_contains", &Bbox::count_contains, "count_contains(xys)\n");
   add_varargs_method("overlaps" , &Bbox::overlaps, "overlaps(bbox)\n");
   add_varargs_method("overlapsx" , &Bbox::overlapsx, "overlapsx(bbox)\n");
   add_varargs_method("overlapsy" , &Bbox::overlapsy, "overlapsy(bbox)\n");
@@ -1359,6 +1795,7 @@ Bbox::init_type()
 
   add_varargs_method("get_bounds", &Bbox::get_bounds, "get_bounds()\n");
   add_varargs_method("update" , &Bbox::update, "update(xys, ignore)\n");
+  add_varargs_method("update_numerix" , &Bbox::update_numerix, "update_numerix(x, u, ignore)\n");
   add_varargs_method("width", 	&Bbox::width, "width()\n");
   add_varargs_method("height", 	&Bbox::height, "height()\n");
   add_varargs_method("xmax", 	&Bbox::xmax, "xmax()\n");
@@ -1393,21 +1830,10 @@ FuncXY::init_type()
   behaviors().doc("Map double,double -> funcx(double), funcy(double)");
   add_varargs_method("map", &FuncXY::map, "map(x,y)\n");
   add_varargs_method("inverse", &FuncXY::inverse, "inverse(x,y)\n");
-  add_varargs_method("set_funcx", &FuncXY::set_funcx, "set_funcx(func)\n");
-  add_varargs_method("set_funcy", &FuncXY::set_funcy, "set_funcy(func)\n");
-  add_varargs_method("get_funcx", &FuncXY::get_funcx, "get_funcx(func)\n");
-  add_varargs_method("get_funcy", &FuncXY::get_funcy, "get_funcy(func)\n");
+  add_varargs_method("set_type", &FuncXY::set_type, "set_type(TYPE)\n");
+  add_varargs_method("get_type", &FuncXY::get_type, "get_type()\n");
 } 
  
-void 
-PolarXY::init_type()
-{
-  _VERBOSE("PolarXY::init_type");
-  behaviors().name("PolarXY");
-  behaviors().doc("map r, theta -> r*cos(theta), r*sin(theta)");
-} 
-
-
 void 
 Transformation::init_type()
 {
@@ -1427,20 +1853,26 @@ Transformation::init_type()
 
   add_varargs_method("get_funcx",   &Transformation::get_funcx,  "get_funcx(); return the Func instance on x\n");
   add_varargs_method("get_funcy",   &Transformation::get_funcy,  "get_funcy(); return the Func instance on y\n");
-
+ 
   add_varargs_method("set_funcx",   &Transformation::set_funcx,  "set_funcx(); set the Func instance on x\n");
   add_varargs_method("set_funcy",   &Transformation::set_funcy,  "set_funcy(); set the Func instance on y\n");
  
 
+  add_varargs_method("get_funcxy",   &Transformation::get_funcxy,  "get_funcxy(); return the FuncXY instance\n");
+  add_varargs_method("set_funcxy",   &Transformation::set_funcxy,  "set_funcxy(); set the FuncXY instance\n");
+
   add_varargs_method("xy_tup",   &Transformation::xy_tup,  "xy_tup(xy)\n");
   add_varargs_method("seq_x_y",  &Transformation::seq_x_y, "seq_x_y(x, y)\n");
   add_varargs_method("numerix_x_y",  &Transformation::numerix_x_y, "numerix_x_y(x, y)\n");
+  add_varargs_method("nonlinear_only_numerix",  &Transformation::nonlinear_only_numerix, "nonlinear_only_numerix\n");
+  add_varargs_method("need_nonlinear",  &Transformation::need_nonlinear, "need_nonlinear\n");
   add_varargs_method("seq_xy_tups", &Transformation::seq_xy_tups, "seq_xy_tups(seq)\n");  
   add_varargs_method("inverse_xy_tup",   &Transformation::inverse_xy_tup,  "inverse_xy_tup(xy)\n");
 
   add_varargs_method("set_offset",   &Transformation::set_offset,  "set_offset(xy, trans)\n");
 
   add_varargs_method("as_vec6", &Transformation::as_vec6, "as_vec6(): return the affine as length 6 list of Values\n");
+  add_varargs_method("as_vec6_val", &Transformation::as_vec6_val, "as_vec6_val(): return the affine as length 6 list of float\n");
   add_varargs_method("deepcopy", &Transformation::deepcopy, "deepcopy()\n");
 
 }
@@ -1463,26 +1895,40 @@ SeparableTransformation::init_type()
 
 }
 
+void 
+NonseparableTransformation::init_type()
+{
+  _VERBOSE("NonseparableTransformation::init_type");
+  behaviors().name("NonseparableTransformation");
+  behaviors().doc("NonseparableTransformation(box1, box2, funcxy); x and y transformations are not independent");
+
+}
+
  
 
 
 
 extern "C"
 DL_EXPORT(void)
-  init_transforms(void)
+#if defined(NUMARRAY)
+  init_na_transforms(void)
+#else
+  init_nc_transforms(void)
+#endif
 {
   static _transforms_module* _transforms = new _transforms_module;
 
-  import_array();  
+#if defined(NUMARRAY)
+  _VERBOSE("init_na_transforms");
+#else
+  _VERBOSE("init_nc_transforms");
+#endif
 
+  import_array();  
 
   Py::Dict d = _transforms->moduleDictionary();
   d["LOG10"] = Py::Int((int)Func::LOG10);
   d["IDENTITY"] = Py::Int((int)Func::IDENTITY);
+  d["POLAR"] = Py::Int((int)FuncXY::POLAR);;
 };
-
-
-
-
-
-
+ 
